"""
Python module generated from Java source file java.util.stream.Stream

Java source file obtained from artifact jdk version jdk

Because this Python module is automatically generated, it may contain errors
and/or code that cannot be parsed. Please report these issues at
https://github.com/magicmq/docs-translator/issues
"""
from java.nio.file import Files
from java.nio.file import Path
from java.util import *
from java.util.concurrent import ConcurrentHashMap
from java.util.function import BiConsumer
from java.util.function import BiFunction
from java.util.function import BinaryOperator
from java.util.function import Consumer
from java.util.function import DoubleConsumer
from java.util.function import Function
from java.util.function import IntConsumer
from java.util.function import IntFunction
from java.util.function import LongConsumer
from java.util.function import Predicate
from java.util.function import Supplier
from java.util.function import ToDoubleFunction
from java.util.function import ToIntFunction
from java.util.function import ToLongFunction
from java.util.function import UnaryOperator
from java.util.stream import *
from typing import Any, Callable, Iterable, Tuple


class Stream(BaseStream):
    """
    A sequence of elements supporting sequential and parallel aggregate
    operations.  The following example illustrates an aggregate operation using
    Stream and IntStream:
    
    ````int sum = widgets.stream()
                         .filter(w -> w.getColor() == RED)
                         .mapToInt(w -> w.getWeight())
                         .sum();````
    
    In this example, `widgets` is a `Collection<Widget>`.  We create
    a stream of `Widget` objects via Collection.stream Collection.stream(),
    filter it to produce a stream containing only the red widgets, and then
    transform it into a stream of `int` values representing the weight of
    each red widget. Then this stream is summed to produce a total weight.
    
    In addition to `Stream`, which is a stream of object references,
    there are primitive specializations for IntStream, LongStream,
    and DoubleStream, all of which are referred to as "streams" and
    conform to the characteristics and restrictions described here.
    
    To perform a computation, stream
    <a href="package-summary.html#StreamOps">operations</a> are composed into a
    *stream pipeline*.  A stream pipeline consists of a source (which
    might be an array, a collection, a generator function, an I/O channel,
    etc), zero or more *intermediate operations* (which transform a
    stream into another stream, such as Stream.filter(Predicate)), and a
    *terminal operation* (which produces a result or side-effect, such
    as Stream.count() or Stream.forEach(Consumer)).
    Streams are lazy; computation on the source data is only performed when the
    terminal operation is initiated, and source elements are consumed only
    as needed.
    
    A stream implementation is permitted significant latitude in optimizing
    the computation of the result.  For example, a stream implementation is free
    to elide operations (or entire stages) from a stream pipeline -- and
    therefore elide invocation of behavioral parameters -- if it can prove that
    it would not affect the result of the computation.  This means that
    side-effects of behavioral parameters may not always be executed and should
    not be relied upon, unless otherwise specified (such as by the terminal
    operations `forEach` and `forEachOrdered`). (For a specific
    example of such an optimization, see the API note documented on the
    .count operation.  For more detail, see the
    <a href="package-summary.html#SideEffects">side-effects</a> section of the
    stream package documentation.)
    
    Collections and streams, while bearing some superficial similarities,
    have different goals.  Collections are primarily concerned with the efficient
    management of, and access to, their elements.  By contrast, streams do not
    provide a means to directly access or manipulate their elements, and are
    instead concerned with declaratively describing their source and the
    computational operations which will be performed in aggregate on that source.
    However, if the provided stream operations do not offer the desired
    functionality, the .iterator() and .spliterator() operations
    can be used to perform a controlled traversal.
    
    A stream pipeline, like the "widgets" example above, can be viewed as
    a *query* on the stream source.  Unless the source was explicitly
    designed for concurrent modification (such as a ConcurrentHashMap),
    unpredictable or erroneous behavior may result from modifying the stream
    source while it is being queried.
    
    Most stream operations accept parameters that describe user-specified
    behavior, such as the lambda expression `w -> w.getWeight()` passed to
    `mapToInt` in the example above.  To preserve correct behavior,
    these *behavioral parameters*:
    
    - must be <a href="package-summary.html#NonInterference">non-interfering</a>
    (they do not modify the stream source); and
    - in most cases must be <a href="package-summary.html#Statelessness">stateless</a>
    (their result should not depend on any state that might change during execution
    of the stream pipeline).
    
    
    Such parameters are always instances of a
    <a href="../function/package-summary.html">functional interface</a> such
    as java.util.function.Function, and are often lambda expressions or
    method references.  Unless otherwise specified these parameters must be
    *non-null*.
    
    A stream should be operated on (invoking an intermediate or terminal stream
    operation) only once.  This rules out, for example, "forked" streams, where
    the same source feeds two or more pipelines, or multiple traversals of the
    same stream.  A stream implementation may throw IllegalStateException
    if it detects that the stream is being reused. However, since some stream
    operations may return their receiver rather than a new stream object, it may
    not be possible to detect reuse in all cases.
    
    Streams have a .close() method and implement AutoCloseable.
    Operating on a stream after it has been closed will throw IllegalStateException.
    Most stream instances do not actually need to be closed after use, as they
    are backed by collections, arrays, or generating functions, which require no
    special resource management. Generally, only streams whose source is an IO channel,
    such as those returned by Files.lines(Path), will require closing. If a
    stream does require closing, it must be opened as a resource within a try-with-resources
    statement or similar control structure to ensure that it is closed promptly after its
    operations have completed.
    
    Stream pipelines may execute either sequentially or in
    <a href="package-summary.html#Parallelism">parallel</a>.  This
    execution mode is a property of the stream.  Streams are created
    with an initial choice of sequential or parallel execution.  (For example,
    Collection.stream() Collection.stream() creates a sequential stream,
    and Collection.parallelStream() Collection.parallelStream() creates
    a parallel one.)  This choice of execution mode may be modified by the
    .sequential() or .parallel() methods, and may be queried with
    the .isParallel() method.
    
    Type `<T>`: the type of the stream elements

    See
    - <a href="package-summary.html">java.util.stream</a>

    Since
    - 1.8
    """

    def filter(self, predicate: "Predicate"["T"]) -> "Stream"["T"]:
        """
        Returns a stream consisting of the elements of this stream that match
        the given predicate.
        
        This is an <a href="package-summary.html#StreamOps">intermediate
        operation</a>.

        Arguments
        - predicate: a <a href="package-summary.html#NonInterference">non-interfering</a>,
                         <a href="package-summary.html#Statelessness">stateless</a>
                         predicate to apply to each element to determine if it
                         should be included

        Returns
        - the new stream
        """
        ...


    def map(self, mapper: "Function"["T", "R"]) -> "Stream"["R"]:
        """
        Returns a stream consisting of the results of applying the given
        function to the elements of this stream.
        
        This is an <a href="package-summary.html#StreamOps">intermediate
        operation</a>.
        
        Type `<R>`: The element type of the new stream

        Arguments
        - mapper: a <a href="package-summary.html#NonInterference">non-interfering</a>,
                      <a href="package-summary.html#Statelessness">stateless</a>
                      function to apply to each element

        Returns
        - the new stream
        """
        ...


    def mapToInt(self, mapper: "ToIntFunction"["T"]) -> "IntStream":
        """
        Returns an `IntStream` consisting of the results of applying the
        given function to the elements of this stream.
        
        This is an <a href="package-summary.html#StreamOps">
            intermediate operation</a>.

        Arguments
        - mapper: a <a href="package-summary.html#NonInterference">non-interfering</a>,
                      <a href="package-summary.html#Statelessness">stateless</a>
                      function to apply to each element

        Returns
        - the new stream
        """
        ...


    def mapToLong(self, mapper: "ToLongFunction"["T"]) -> "LongStream":
        """
        Returns a `LongStream` consisting of the results of applying the
        given function to the elements of this stream.
        
        This is an <a href="package-summary.html#StreamOps">intermediate
        operation</a>.

        Arguments
        - mapper: a <a href="package-summary.html#NonInterference">non-interfering</a>,
                      <a href="package-summary.html#Statelessness">stateless</a>
                      function to apply to each element

        Returns
        - the new stream
        """
        ...


    def mapToDouble(self, mapper: "ToDoubleFunction"["T"]) -> "DoubleStream":
        """
        Returns a `DoubleStream` consisting of the results of applying the
        given function to the elements of this stream.
        
        This is an <a href="package-summary.html#StreamOps">intermediate
        operation</a>.

        Arguments
        - mapper: a <a href="package-summary.html#NonInterference">non-interfering</a>,
                      <a href="package-summary.html#Statelessness">stateless</a>
                      function to apply to each element

        Returns
        - the new stream
        """
        ...


    def flatMap(self, mapper: "Function"["T", "Stream"["R"]]) -> "Stream"["R"]:
        """
        Returns a stream consisting of the results of replacing each element of
        this stream with the contents of a mapped stream produced by applying
        the provided mapping function to each element.  Each mapped stream is
        java.util.stream.BaseStream.close() closed after its contents
        have been placed into this stream.  (If a mapped stream is `null`
        an empty stream is used, instead.)
        
        This is an <a href="package-summary.html#StreamOps">intermediate
        operation</a>.
        
        Type `<R>`: The element type of the new stream

        Arguments
        - mapper: a <a href="package-summary.html#NonInterference">non-interfering</a>,
                      <a href="package-summary.html#Statelessness">stateless</a>
                      function to apply to each element which produces a stream
                      of new values

        Returns
        - the new stream

        See
        - .mapMulti

        Unknown Tags
        - The `flatMap()` operation has the effect of applying a one-to-many
        transformation to the elements of the stream, and then flattening the
        resulting elements into a new stream.
        
        **Examples.**
        
        If `orders` is a stream of purchase orders, and each purchase
        order contains a collection of line items, then the following produces a
        stream containing all the line items in all the orders:
        ````orders.flatMap(order -> order.getLineItems().stream())...````
        
        If `path` is the path to a file, then the following produces a
        stream of the `words` contained in that file:
        ````Stream<String> lines = Files.lines(path, StandardCharsets.UTF_8);
            Stream<String> words = lines.flatMap(line -> Stream.of(line.split(" +")));````
        The `mapper` function passed to `flatMap` splits a line,
        using a simple regular expression, into an array of words, and then
        creates a stream of words from that array.
        """
        ...


    def flatMapToInt(self, mapper: "Function"["T", "IntStream"]) -> "IntStream":
        """
        Returns an `IntStream` consisting of the results of replacing each
        element of this stream with the contents of a mapped stream produced by
        applying the provided mapping function to each element.  Each mapped
        stream is java.util.stream.BaseStream.close() closed after its
        contents have been placed into this stream.  (If a mapped stream is
        `null` an empty stream is used, instead.)
        
        This is an <a href="package-summary.html#StreamOps">intermediate
        operation</a>.

        Arguments
        - mapper: a <a href="package-summary.html#NonInterference">non-interfering</a>,
                      <a href="package-summary.html#Statelessness">stateless</a>
                      function to apply to each element which produces a stream
                      of new values

        Returns
        - the new stream

        See
        - .flatMap(Function)
        """
        ...


    def flatMapToLong(self, mapper: "Function"["T", "LongStream"]) -> "LongStream":
        """
        Returns an `LongStream` consisting of the results of replacing each
        element of this stream with the contents of a mapped stream produced by
        applying the provided mapping function to each element.  Each mapped
        stream is java.util.stream.BaseStream.close() closed after its
        contents have been placed into this stream.  (If a mapped stream is
        `null` an empty stream is used, instead.)
        
        This is an <a href="package-summary.html#StreamOps">intermediate
        operation</a>.

        Arguments
        - mapper: a <a href="package-summary.html#NonInterference">non-interfering</a>,
                      <a href="package-summary.html#Statelessness">stateless</a>
                      function to apply to each element which produces a stream
                      of new values

        Returns
        - the new stream

        See
        - .flatMap(Function)
        """
        ...


    def flatMapToDouble(self, mapper: "Function"["T", "DoubleStream"]) -> "DoubleStream":
        """
        Returns an `DoubleStream` consisting of the results of replacing
        each element of this stream with the contents of a mapped stream produced
        by applying the provided mapping function to each element.  Each mapped
        stream is java.util.stream.BaseStream.close() closed after its
        contents have placed been into this stream.  (If a mapped stream is
        `null` an empty stream is used, instead.)
        
        This is an <a href="package-summary.html#StreamOps">intermediate
        operation</a>.

        Arguments
        - mapper: a <a href="package-summary.html#NonInterference">non-interfering</a>,
                      <a href="package-summary.html#Statelessness">stateless</a>
                      function to apply to each element which produces a stream
                      of new values

        Returns
        - the new stream

        See
        - .flatMap(Function)
        """
        ...


    def mapMulti(self, mapper: "BiConsumer"["T", "Consumer"["R"]]) -> "Stream"["R"]:
        """
        Returns a stream consisting of the results of replacing each element of
        this stream with multiple elements, specifically zero or more elements.
        Replacement is performed by applying the provided mapping function to each
        element in conjunction with a Consumer consumer argument
        that accepts replacement elements. The mapping function calls the consumer
        zero or more times to provide the replacement elements.
        
        This is an <a href="package-summary.html#StreamOps">intermediate
        operation</a>.
        
        If the Consumer consumer argument is used outside the scope of
        its application to the mapping function, the results are undefined.
        
        Type `<R>`: The element type of the new stream

        Arguments
        - mapper: a <a href="package-summary.html#NonInterference">non-interfering</a>,
                      <a href="package-summary.html#Statelessness">stateless</a>
                      function that generates replacement elements

        Returns
        - the new stream

        See
        - .flatMap flatMap

        Since
        - 16

        Unknown Tags
        - The default implementation invokes .flatMap flatMap on this stream,
        passing a function that behaves as follows. First, it calls the mapper function
        with a `Consumer` that accumulates replacement elements into a newly created
        internal buffer. When the mapper function returns, it creates a stream from the
        internal buffer. Finally, it returns this stream to `flatMap`.
        - This method is similar to .flatMap flatMap in that it applies a one-to-many
        transformation to the elements of the stream and flattens the result elements
        into a new stream. This method is preferable to `flatMap` in the following
        circumstances:
        
        - When replacing each stream element with a small (possibly zero) number of
        elements. Using this method avoids the overhead of creating a new Stream instance
        for every group of result elements, as required by `flatMap`.
        - When it is easier to use an imperative approach for generating result
        elements than it is to return them in the form of a Stream.
        
        
        If a lambda expression is provided as the mapper function argument, additional type
        information may be necessary for proper inference of the element type `<R>` of
        the returned stream. This can be provided in the form of explicit type declarations for
        the lambda parameters or as an explicit type argument to the `mapMulti` call.
        
        **Examples**
        
        Given a stream of `Number` objects, the following
        produces a list containing only the `Integer` objects:
        ````Stream<Number> numbers = ... ;
            List<Integer> integers = numbers.<Integer>mapMulti((number, consumer) -> {
                    if (number instanceof Integer i)
                        consumer.accept(i);`)
                .collect(Collectors.toList());
        }```
        
        If we have an `Iterable<Object>` and need to recursively expand its elements
        that are themselves of type `Iterable`, we can use `mapMulti` as follows:
        ````class C {
            static void expandIterable(Object e, Consumer<Object> c) {
                if (e instanceof Iterable<?> elements) {
                    for (Object ie : elements) {
                        expandIterable(ie, c);`
                } else if (e != null) {
                    c.accept(e);
                }
            }
        
            public static void main(String[] args) {
                var nestedList = List.of(1, List.of(2, List.of(3, 4)), 5);
                Stream<Object> expandedStream = nestedList.stream().mapMulti(C::expandIterable);
            }
        }
        }```
        """
        ...


    def mapMultiToInt(self, mapper: "BiConsumer"["T", "IntConsumer"]) -> "IntStream":
        """
        Returns an `IntStream` consisting of the results of replacing each
        element of this stream with multiple elements, specifically zero or more
        elements.
        Replacement is performed by applying the provided mapping function to each
        element in conjunction with a IntConsumer consumer argument
        that accepts replacement elements. The mapping function calls the consumer
        zero or more times to provide the replacement elements.
        
        This is an <a href="package-summary.html#StreamOps">intermediate
        operation</a>.
        
        If the IntConsumer consumer argument is used outside the scope of
        its application to the mapping function, the results are undefined.

        Arguments
        - mapper: a <a href="package-summary.html#NonInterference">non-interfering</a>,
                      <a href="package-summary.html#Statelessness">stateless</a>
                      function that generates replacement elements

        Returns
        - the new stream

        See
        - .mapMulti mapMulti

        Since
        - 16

        Unknown Tags
        - The default implementation invokes .flatMapToInt flatMapToInt on this stream,
        passing a function that behaves as follows. First, it calls the mapper function
        with an `IntConsumer` that accumulates replacement elements into a newly created
        internal buffer. When the mapper function returns, it creates an `IntStream` from
        the internal buffer. Finally, it returns this stream to `flatMapToInt`.
        """
        ...


    def mapMultiToLong(self, mapper: "BiConsumer"["T", "LongConsumer"]) -> "LongStream":
        """
        Returns a `LongStream` consisting of the results of replacing each
        element of this stream with multiple elements, specifically zero or more
        elements.
        Replacement is performed by applying the provided mapping function to each
        element in conjunction with a LongConsumer consumer argument
        that accepts replacement elements. The mapping function calls the consumer
        zero or more times to provide the replacement elements.
        
        This is an <a href="package-summary.html#StreamOps">intermediate
        operation</a>.
        
        If the LongConsumer consumer argument is used outside the scope of
        its application to the mapping function, the results are undefined.

        Arguments
        - mapper: a <a href="package-summary.html#NonInterference">non-interfering</a>,
                      <a href="package-summary.html#Statelessness">stateless</a>
                      function that generates replacement elements

        Returns
        - the new stream

        See
        - .mapMulti mapMulti

        Since
        - 16

        Unknown Tags
        - The default implementation invokes .flatMapToLong flatMapToLong on this stream,
        passing a function that behaves as follows. First, it calls the mapper function
        with a `LongConsumer` that accumulates replacement elements into a newly created
        internal buffer. When the mapper function returns, it creates a `LongStream` from
        the internal buffer. Finally, it returns this stream to `flatMapToLong`.
        """
        ...


    def mapMultiToDouble(self, mapper: "BiConsumer"["T", "DoubleConsumer"]) -> "DoubleStream":
        """
        Returns a `DoubleStream` consisting of the results of replacing each
        element of this stream with multiple elements, specifically zero or more
        elements.
        Replacement is performed by applying the provided mapping function to each
        element in conjunction with a DoubleConsumer consumer argument
        that accepts replacement elements. The mapping function calls the consumer
        zero or more times to provide the replacement elements.
        
        This is an <a href="package-summary.html#StreamOps">intermediate
        operation</a>.
        
        If the DoubleConsumer consumer argument is used outside the scope of
        its application to the mapping function, the results are undefined.

        Arguments
        - mapper: a <a href="package-summary.html#NonInterference">non-interfering</a>,
                      <a href="package-summary.html#Statelessness">stateless</a>
                      function that generates replacement elements

        Returns
        - the new stream

        See
        - .mapMulti mapMulti

        Since
        - 16

        Unknown Tags
        - The default implementation invokes .flatMapToDouble flatMapToDouble on this stream,
        passing a function that behaves as follows. First, it calls the mapper function
        with an `DoubleConsumer` that accumulates replacement elements into a newly created
        internal buffer. When the mapper function returns, it creates a `DoubleStream` from
        the internal buffer. Finally, it returns this stream to `flatMapToDouble`.
        """
        ...


    def distinct(self) -> "Stream"["T"]:
        """
        Returns a stream consisting of the distinct elements (according to
        Object.equals(Object)) of this stream.
        
        For ordered streams, the selection of distinct elements is stable
        (for duplicated elements, the element appearing first in the encounter
        order is preserved.)  For unordered streams, no stability guarantees
        are made.
        
        This is a <a href="package-summary.html#StreamOps">stateful
        intermediate operation</a>.

        Returns
        - the new stream

        Unknown Tags
        - Preserving stability for `distinct()` in parallel pipelines is
        relatively expensive (requires that the operation act as a full barrier,
        with substantial buffering overhead), and stability is often not needed.
        Using an unordered stream source (such as .generate(Supplier))
        or removing the ordering constraint with .unordered() may result
        in significantly more efficient execution for `distinct()` in parallel
        pipelines, if the semantics of your situation permit.  If consistency
        with encounter order is required, and you are experiencing poor performance
        or memory utilization with `distinct()` in parallel pipelines,
        switching to sequential execution with .sequential() may improve
        performance.
        """
        ...


    def sorted(self) -> "Stream"["T"]:
        """
        Returns a stream consisting of the elements of this stream, sorted
        according to natural order.  If the elements of this stream are not
        `Comparable`, a `java.lang.ClassCastException` may be thrown
        when the terminal operation is executed.
        
        For ordered streams, the sort is stable.  For unordered streams, no
        stability guarantees are made.
        
        This is a <a href="package-summary.html#StreamOps">stateful
        intermediate operation</a>.

        Returns
        - the new stream
        """
        ...


    def sorted(self, comparator: "Comparator"["T"]) -> "Stream"["T"]:
        """
        Returns a stream consisting of the elements of this stream, sorted
        according to the provided `Comparator`.
        
        For ordered streams, the sort is stable.  For unordered streams, no
        stability guarantees are made.
        
        This is a <a href="package-summary.html#StreamOps">stateful
        intermediate operation</a>.

        Arguments
        - comparator: a <a href="package-summary.html#NonInterference">non-interfering</a>,
                          <a href="package-summary.html#Statelessness">stateless</a>
                          `Comparator` to be used to compare stream elements

        Returns
        - the new stream
        """
        ...


    def peek(self, action: "Consumer"["T"]) -> "Stream"["T"]:
        """
        Returns a stream consisting of the elements of this stream, additionally
        performing the provided action on each element as elements are consumed
        from the resulting stream.
        
        This is an <a href="package-summary.html#StreamOps">intermediate
        operation</a>.
        
        For parallel stream pipelines, the action may be called at
        whatever time and in whatever thread the element is made available by the
        upstream operation.  If the action modifies shared state,
        it is responsible for providing the required synchronization.

        Arguments
        - action: a <a href="package-summary.html#NonInterference">
                        non-interfering</a> action to perform on the elements as
                        they are consumed from the stream

        Returns
        - the new stream

        Unknown Tags
        - This method exists mainly to support debugging, where you want
        to see the elements as they flow past a certain point in a pipeline:
        ````Stream.of("one", "two", "three", "four")
                .filter(e -> e.length() > 3)
                .peek(e -> System.out.println("Filtered value: " + e))
                .map(String::toUpperCase)
                .peek(e -> System.out.println("Mapped value: " + e))
                .collect(Collectors.toList());````
        
        In cases where the stream implementation is able to optimize away the
        production of some or all the elements (such as with short-circuiting
        operations like `findFirst`, or in the example described in
        .count), the action will not be invoked for those elements.
        """
        ...


    def limit(self, maxSize: int) -> "Stream"["T"]:
        """
        Returns a stream consisting of the elements of this stream, truncated
        to be no longer than `maxSize` in length.
        
        This is a <a href="package-summary.html#StreamOps">short-circuiting
        stateful intermediate operation</a>.

        Arguments
        - maxSize: the number of elements the stream should be limited to

        Returns
        - the new stream

        Raises
        - IllegalArgumentException: if `maxSize` is negative

        Unknown Tags
        - While `limit()` is generally a cheap operation on sequential
        stream pipelines, it can be quite expensive on ordered parallel pipelines,
        especially for large values of `maxSize`, since `limit(n)`
        is constrained to return not just any *n* elements, but the
        *first n* elements in the encounter order.  Using an unordered
        stream source (such as .generate(Supplier)) or removing the
        ordering constraint with .unordered() may result in significant
        speedups of `limit()` in parallel pipelines, if the semantics of
        your situation permit.  If consistency with encounter order is required,
        and you are experiencing poor performance or memory utilization with
        `limit()` in parallel pipelines, switching to sequential execution
        with .sequential() may improve performance.
        """
        ...


    def skip(self, n: int) -> "Stream"["T"]:
        """
        Returns a stream consisting of the remaining elements of this stream
        after discarding the first `n` elements of the stream.
        If this stream contains fewer than `n` elements then an
        empty stream will be returned.
        
        This is a <a href="package-summary.html#StreamOps">stateful
        intermediate operation</a>.

        Arguments
        - n: the number of leading elements to skip

        Returns
        - the new stream

        Raises
        - IllegalArgumentException: if `n` is negative

        Unknown Tags
        - While `skip()` is generally a cheap operation on sequential
        stream pipelines, it can be quite expensive on ordered parallel pipelines,
        especially for large values of `n`, since `skip(n)`
        is constrained to skip not just any *n* elements, but the
        *first n* elements in the encounter order.  Using an unordered
        stream source (such as .generate(Supplier)) or removing the
        ordering constraint with .unordered() may result in significant
        speedups of `skip()` in parallel pipelines, if the semantics of
        your situation permit.  If consistency with encounter order is required,
        and you are experiencing poor performance or memory utilization with
        `skip()` in parallel pipelines, switching to sequential execution
        with .sequential() may improve performance.
        """
        ...


    def takeWhile(self, predicate: "Predicate"["T"]) -> "Stream"["T"]:
        """
        Returns, if this stream is ordered, a stream consisting of the longest
        prefix of elements taken from this stream that match the given predicate.
        Otherwise returns, if this stream is unordered, a stream consisting of a
        subset of elements taken from this stream that match the given predicate.
        
        If this stream is ordered then the longest prefix is a contiguous
        sequence of elements of this stream that match the given predicate.  The
        first element of the sequence is the first element of this stream, and
        the element immediately following the last element of the sequence does
        not match the given predicate.
        
        If this stream is unordered, and some (but not all) elements of this
        stream match the given predicate, then the behavior of this operation is
        nondeterministic; it is free to take any subset of matching elements
        (which includes the empty set).
        
        Independent of whether this stream is ordered or unordered if all
        elements of this stream match the given predicate then this operation
        takes all elements (the result is the same as the input), or if no
        elements of the stream match the given predicate then no elements are
        taken (the result is an empty stream).
        
        This is a <a href="package-summary.html#StreamOps">short-circuiting
        stateful intermediate operation</a>.

        Arguments
        - predicate: a <a href="package-summary.html#NonInterference">non-interfering</a>,
                         <a href="package-summary.html#Statelessness">stateless</a>
                         predicate to apply to elements to determine the longest
                         prefix of elements.

        Returns
        - the new stream

        Since
        - 9

        Unknown Tags
        - The default implementation obtains the .spliterator() spliterator
        of this stream, wraps that spliterator so as to support the semantics
        of this operation on traversal, and returns a new stream associated with
        the wrapped spliterator.  The returned stream preserves the execution
        characteristics of this stream (namely parallel or sequential execution
        as per .isParallel()) but the wrapped spliterator may choose to
        not support splitting.  When the returned stream is closed, the close
        handlers for both the returned and this stream are invoked.
        - While `takeWhile()` is generally a cheap operation on sequential
        stream pipelines, it can be quite expensive on ordered parallel
        pipelines, since the operation is constrained to return not just any
        valid prefix, but the longest prefix of elements in the encounter order.
        Using an unordered stream source (such as .generate(Supplier)) or
        removing the ordering constraint with .unordered() may result in
        significant speedups of `takeWhile()` in parallel pipelines, if the
        semantics of your situation permit.  If consistency with encounter order
        is required, and you are experiencing poor performance or memory
        utilization with `takeWhile()` in parallel pipelines, switching to
        sequential execution with .sequential() may improve performance.
        """
        ...


    def dropWhile(self, predicate: "Predicate"["T"]) -> "Stream"["T"]:
        """
        Returns, if this stream is ordered, a stream consisting of the remaining
        elements of this stream after dropping the longest prefix of elements
        that match the given predicate.  Otherwise returns, if this stream is
        unordered, a stream consisting of the remaining elements of this stream
        after dropping a subset of elements that match the given predicate.
        
        If this stream is ordered then the longest prefix is a contiguous
        sequence of elements of this stream that match the given predicate.  The
        first element of the sequence is the first element of this stream, and
        the element immediately following the last element of the sequence does
        not match the given predicate.
        
        If this stream is unordered, and some (but not all) elements of this
        stream match the given predicate, then the behavior of this operation is
        nondeterministic; it is free to drop any subset of matching elements
        (which includes the empty set).
        
        Independent of whether this stream is ordered or unordered if all
        elements of this stream match the given predicate then this operation
        drops all elements (the result is an empty stream), or if no elements of
        the stream match the given predicate then no elements are dropped (the
        result is the same as the input).
        
        This is a <a href="package-summary.html#StreamOps">stateful
        intermediate operation</a>.

        Arguments
        - predicate: a <a href="package-summary.html#NonInterference">non-interfering</a>,
                         <a href="package-summary.html#Statelessness">stateless</a>
                         predicate to apply to elements to determine the longest
                         prefix of elements.

        Returns
        - the new stream

        Since
        - 9

        Unknown Tags
        - The default implementation obtains the .spliterator() spliterator
        of this stream, wraps that spliterator so as to support the semantics
        of this operation on traversal, and returns a new stream associated with
        the wrapped spliterator.  The returned stream preserves the execution
        characteristics of this stream (namely parallel or sequential execution
        as per .isParallel()) but the wrapped spliterator may choose to
        not support splitting.  When the returned stream is closed, the close
        handlers for both the returned and this stream are invoked.
        - While `dropWhile()` is generally a cheap operation on sequential
        stream pipelines, it can be quite expensive on ordered parallel
        pipelines, since the operation is constrained to return not just any
        valid prefix, but the longest prefix of elements in the encounter order.
        Using an unordered stream source (such as .generate(Supplier)) or
        removing the ordering constraint with .unordered() may result in
        significant speedups of `dropWhile()` in parallel pipelines, if the
        semantics of your situation permit.  If consistency with encounter order
        is required, and you are experiencing poor performance or memory
        utilization with `dropWhile()` in parallel pipelines, switching to
        sequential execution with .sequential() may improve performance.
        """
        ...


    def forEach(self, action: "Consumer"["T"]) -> None:
        """
        Performs an action for each element of this stream.
        
        This is a <a href="package-summary.html#StreamOps">terminal
        operation</a>.
        
        The behavior of this operation is explicitly nondeterministic.
        For parallel stream pipelines, this operation does *not*
        guarantee to respect the encounter order of the stream, as doing so
        would sacrifice the benefit of parallelism.  For any given element, the
        action may be performed at whatever time and in whatever thread the
        library chooses.  If the action accesses shared state, it is
        responsible for providing the required synchronization.

        Arguments
        - action: a <a href="package-summary.html#NonInterference">
                      non-interfering</a> action to perform on the elements
        """
        ...


    def forEachOrdered(self, action: "Consumer"["T"]) -> None:
        """
        Performs an action for each element of this stream, in the encounter
        order of the stream if the stream has a defined encounter order.
        
        This is a <a href="package-summary.html#StreamOps">terminal
        operation</a>.
        
        This operation processes the elements one at a time, in encounter
        order if one exists.  Performing the action for one element
        <a href="../concurrent/package-summary.html#MemoryVisibility">*happens-before*</a>
        performing the action for subsequent elements, but for any given element,
        the action may be performed in whatever thread the library chooses.

        Arguments
        - action: a <a href="package-summary.html#NonInterference">
                      non-interfering</a> action to perform on the elements

        See
        - .forEach(Consumer)
        """
        ...


    def toArray(self) -> list["Object"]:
        """
        Returns an array containing the elements of this stream.
        
        This is a <a href="package-summary.html#StreamOps">terminal
        operation</a>.

        Returns
        - an array, whose Class.getComponentType runtime component
        type is `Object`, containing the elements of this stream
        """
        ...


    def toArray(self, generator: "IntFunction"[list["A"]]) -> list["A"]:
        """
        Returns an array containing the elements of this stream, using the
        provided `generator` function to allocate the returned array, as
        well as any additional arrays that might be required for a partitioned
        execution or for resizing.
        
        This is a <a href="package-summary.html#StreamOps">terminal
        operation</a>.
        
        Type `<A>`: the component type of the resulting array

        Arguments
        - generator: a function which produces a new array of the desired
                         type and the provided length

        Returns
        - an array containing the elements in this stream

        Raises
        - ArrayStoreException: if the runtime type of any element of this
                stream is not assignable to the Class.getComponentType
                runtime component type of the generated array

        Unknown Tags
        - The generator function takes an integer, which is the size of the
        desired array, and produces an array of the desired size.  This can be
        concisely expressed with an array constructor reference:
        ````Person[] men = people.stream()
                                 .filter(p -> p.getGender() == MALE)
                                 .toArray(Person[]::new);````
        """
        ...


    def reduce(self, identity: "T", accumulator: "BinaryOperator"["T"]) -> "T":
        """
        Performs a <a href="package-summary.html#Reduction">reduction</a> on the
        elements of this stream, using the provided identity value and an
        <a href="package-summary.html#Associativity">associative</a>
        accumulation function, and returns the reduced value.  This is equivalent
        to:
        ````T result = identity;
            for (T element : this stream)
                result = accumulator.apply(result, element)
            return result;````
        
        but is not constrained to execute sequentially.
        
        The `identity` value must be an identity for the accumulator
        function. This means that for all `t`,
        `accumulator.apply(identity, t)` is equal to `t`.
        The `accumulator` function must be an
        <a href="package-summary.html#Associativity">associative</a> function.
        
        This is a <a href="package-summary.html#StreamOps">terminal
        operation</a>.

        Arguments
        - identity: the identity value for the accumulating function
        - accumulator: an <a href="package-summary.html#Associativity">associative</a>,
                           <a href="package-summary.html#NonInterference">non-interfering</a>,
                           <a href="package-summary.html#Statelessness">stateless</a>
                           function for combining two values

        Returns
        - the result of the reduction

        Unknown Tags
        - Sum, min, max, average, and string concatenation are all special
        cases of reduction. Summing a stream of numbers can be expressed as:
        
        ````Integer sum = integers.reduce(0, (a, b) -> a+b);````
        
        or:
        
        ````Integer sum = integers.reduce(0, Integer::sum);````
        
        While this may seem a more roundabout way to perform an aggregation
        compared to simply mutating a running total in a loop, reduction
        operations parallelize more gracefully, without needing additional
        synchronization and with greatly reduced risk of data races.
        """
        ...


    def reduce(self, accumulator: "BinaryOperator"["T"]) -> "Optional"["T"]:
        """
        Performs a <a href="package-summary.html#Reduction">reduction</a> on the
        elements of this stream, using an
        <a href="package-summary.html#Associativity">associative</a> accumulation
        function, and returns an `Optional` describing the reduced value,
        if any. This is equivalent to:
        ````boolean foundAny = False;
            T result = null;
            for (T element : this stream) {
                if (!foundAny) {
                    foundAny = True;
                    result = element;`
                else
                    result = accumulator.apply(result, element);
            }
            return foundAny ? Optional.of(result) : Optional.empty();
        }```
        
        but is not constrained to execute sequentially.
        
        The `accumulator` function must be an
        <a href="package-summary.html#Associativity">associative</a> function.
        
        This is a <a href="package-summary.html#StreamOps">terminal
        operation</a>.

        Arguments
        - accumulator: an <a href="package-summary.html#Associativity">associative</a>,
                           <a href="package-summary.html#NonInterference">non-interfering</a>,
                           <a href="package-summary.html#Statelessness">stateless</a>
                           function for combining two values

        Returns
        - an Optional describing the result of the reduction

        Raises
        - NullPointerException: if the result of the reduction is null

        See
        - .max(Comparator)
        """
        ...


    def reduce(self, identity: "U", accumulator: "BiFunction"["U", "T", "U"], combiner: "BinaryOperator"["U"]) -> "U":
        """
        Performs a <a href="package-summary.html#Reduction">reduction</a> on the
        elements of this stream, using the provided identity, accumulation and
        combining functions.  This is equivalent to:
        ````U result = identity;
            for (T element : this stream)
                result = accumulator.apply(result, element)
            return result;````
        
        but is not constrained to execute sequentially.
        
        The `identity` value must be an identity for the combiner
        function.  This means that for all `u`, `combiner(identity, u)`
        is equal to `u`.  Additionally, the `combiner` function
        must be compatible with the `accumulator` function; for all
        `u` and `t`, the following must hold:
        ````combiner.apply(u, accumulator.apply(identity, t)) == accumulator.apply(u, t)````
        
        This is a <a href="package-summary.html#StreamOps">terminal
        operation</a>.
        
        Type `<U>`: The type of the result

        Arguments
        - identity: the identity value for the combiner function
        - accumulator: an <a href="package-summary.html#Associativity">associative</a>,
                           <a href="package-summary.html#NonInterference">non-interfering</a>,
                           <a href="package-summary.html#Statelessness">stateless</a>
                           function for incorporating an additional element into a result
        - combiner: an <a href="package-summary.html#Associativity">associative</a>,
                           <a href="package-summary.html#NonInterference">non-interfering</a>,
                           <a href="package-summary.html#Statelessness">stateless</a>
                           function for combining two values, which must be
                           compatible with the accumulator function

        Returns
        - the result of the reduction

        See
        - .reduce(Object, BinaryOperator)

        Unknown Tags
        - Many reductions using this form can be represented more simply
        by an explicit combination of `map` and `reduce` operations.
        The `accumulator` function acts as a fused mapper and accumulator,
        which can sometimes be more efficient than separate mapping and reduction,
        such as when knowing the previously reduced value allows you to avoid
        some computation.
        """
        ...


    def collect(self, supplier: "Supplier"["R"], accumulator: "BiConsumer"["R", "T"], combiner: "BiConsumer"["R", "R"]) -> "R":
        """
        Performs a <a href="package-summary.html#MutableReduction">mutable
        reduction</a> operation on the elements of this stream.  A mutable
        reduction is one in which the reduced value is a mutable result container,
        such as an `ArrayList`, and elements are incorporated by updating
        the state of the result rather than by replacing the result.  This
        produces a result equivalent to:
        ````R result = supplier.get();
            for (T element : this stream)
                accumulator.accept(result, element);
            return result;````
        
        Like .reduce(Object, BinaryOperator), `collect` operations
        can be parallelized without requiring additional synchronization.
        
        This is a <a href="package-summary.html#StreamOps">terminal
        operation</a>.
        
        Type `<R>`: the type of the mutable result container

        Arguments
        - supplier: a function that creates a new mutable result container.
                        For a parallel execution, this function may be called
                        multiple times and must return a fresh value each time.
        - accumulator: an <a href="package-summary.html#Associativity">associative</a>,
                           <a href="package-summary.html#NonInterference">non-interfering</a>,
                           <a href="package-summary.html#Statelessness">stateless</a>
                           function that must fold an element into a result
                           container.
        - combiner: an <a href="package-summary.html#Associativity">associative</a>,
                           <a href="package-summary.html#NonInterference">non-interfering</a>,
                           <a href="package-summary.html#Statelessness">stateless</a>
                           function that accepts two partial result containers
                           and merges them, which must be compatible with the
                           accumulator function.  The combiner function must fold
                           the elements from the second result container into the
                           first result container.

        Returns
        - the result of the reduction

        Unknown Tags
        - There are many existing classes in the JDK whose signatures are
        well-suited for use with method references as arguments to `collect()`.
        For example, the following will accumulate strings into an `ArrayList`:
        ````List<String> asList = stringStream.collect(ArrayList::new, ArrayList::add,
                                                       ArrayList::addAll);````
        
        The following will take a stream of strings and concatenates them into a
        single string:
        ````String concat = stringStream.collect(StringBuilder::new, StringBuilder::append,
                                                 StringBuilder::append)
                                        .toString();````
        """
        ...


    def collect(self, collector: "Collector"["T", "A", "R"]) -> "R":
        """
        Performs a <a href="package-summary.html#MutableReduction">mutable
        reduction</a> operation on the elements of this stream using a
        `Collector`.  A `Collector`
        encapsulates the functions used as arguments to
        .collect(Supplier, BiConsumer, BiConsumer), allowing for reuse of
        collection strategies and composition of collect operations such as
        multiple-level grouping or partitioning.
        
        If the stream is parallel, and the `Collector`
        is Collector.Characteristics.CONCURRENT concurrent, and
        either the stream is unordered or the collector is
        Collector.Characteristics.UNORDERED unordered,
        then a concurrent reduction will be performed (see Collector for
        details on concurrent reduction.)
        
        This is a <a href="package-summary.html#StreamOps">terminal
        operation</a>.
        
        When executed in parallel, multiple intermediate results may be
        instantiated, populated, and merged so as to maintain isolation of
        mutable data structures.  Therefore, even when executed in parallel
        with non-thread-safe data structures (such as `ArrayList`), no
        additional synchronization is needed for a parallel reduction.
        
        Type `<R>`: the type of the result
        
        Type `<A>`: the intermediate accumulation type of the `Collector`

        Arguments
        - collector: the `Collector` describing the reduction

        Returns
        - the result of the reduction

        See
        - Collectors

        Unknown Tags
        - The following will accumulate strings into a List:
        ````List<String> asList = stringStream.collect(Collectors.toList());````
        
        The following will classify `Person` objects by city:
        ````Map<String, List<Person>> peopleByCity
                = personStream.collect(Collectors.groupingBy(Person::getCity));````
        
        The following will classify `Person` objects by state and city,
        cascading two `Collector`s together:
        ````Map<String, Map<String, List<Person>>> peopleByStateAndCity
                = personStream.collect(Collectors.groupingBy(Person::getState,
                                                             Collectors.groupingBy(Person::getCity)));````
        """
        ...


    def toList(self) -> list["T"]:
        """
        Accumulates the elements of this stream into a `List`. The elements in
        the list will be in this stream's encounter order, if one exists. The returned List
        is unmodifiable; calls to any mutator method will always cause
        `UnsupportedOperationException` to be thrown. There are no
        guarantees on the implementation type or serializability of the returned List.
        
        The returned instance may be <a href="/java.base/java/lang/doc-files/ValueBased.html">value-based</a>.
        Callers should make no assumptions about the identity of the returned instances.
        Identity-sensitive operations on these instances (reference equality (`==`),
        identity hash code, and synchronization) are unreliable and should be avoided.
        
        This is a <a href="package-summary.html#StreamOps">terminal operation</a>.

        Returns
        - a List containing the stream elements

        Since
        - 16

        Unknown Tags
        - If more control over the returned object is required, use
        Collectors.toCollection(Supplier).
        - The implementation in this interface returns a List produced as if by the following:
        ````Collections.unmodifiableList(new ArrayList<>(Arrays.asList(this.toArray())))````
        - Most instances of Stream will override this method and provide an implementation
        that is highly optimized compared to the implementation in this interface.
        """
        ...


    def min(self, comparator: "Comparator"["T"]) -> "Optional"["T"]:
        """
        Returns the minimum element of this stream according to the provided
        `Comparator`.  This is a special case of a
        <a href="package-summary.html#Reduction">reduction</a>.
        
        This is a <a href="package-summary.html#StreamOps">terminal operation</a>.

        Arguments
        - comparator: a <a href="package-summary.html#NonInterference">non-interfering</a>,
                          <a href="package-summary.html#Statelessness">stateless</a>
                          `Comparator` to compare elements of this stream

        Returns
        - an `Optional` describing the minimum element of this stream,
        or an empty `Optional` if the stream is empty

        Raises
        - NullPointerException: if the minimum element is null
        """
        ...


    def max(self, comparator: "Comparator"["T"]) -> "Optional"["T"]:
        """
        Returns the maximum element of this stream according to the provided
        `Comparator`.  This is a special case of a
        <a href="package-summary.html#Reduction">reduction</a>.
        
        This is a <a href="package-summary.html#StreamOps">terminal
        operation</a>.

        Arguments
        - comparator: a <a href="package-summary.html#NonInterference">non-interfering</a>,
                          <a href="package-summary.html#Statelessness">stateless</a>
                          `Comparator` to compare elements of this stream

        Returns
        - an `Optional` describing the maximum element of this stream,
        or an empty `Optional` if the stream is empty

        Raises
        - NullPointerException: if the maximum element is null
        """
        ...


    def count(self) -> int:
        """
        Returns the count of elements in this stream.  This is a special case of
        a <a href="package-summary.html#Reduction">reduction</a> and is
        equivalent to:
        ````return mapToLong(e -> 1L).sum();````
        
        This is a <a href="package-summary.html#StreamOps">terminal operation</a>.

        Returns
        - the count of elements in this stream

        Unknown Tags
        - An implementation may choose to not execute the stream pipeline (either
        sequentially or in parallel) if it is capable of computing the count
        directly from the stream source.  In such cases no source elements will
        be traversed and no intermediate operations will be evaluated.
        Behavioral parameters with side-effects, which are strongly discouraged
        except for harmless cases such as debugging, may be affected.  For
        example, consider the following stream:
        ````List<String> l = Arrays.asList("A", "B", "C", "D");
            long count = l.stream().peek(System.out::println).count();````
        The number of elements covered by the stream source, a `List`, is
        known and the intermediate operation, `peek`, does not inject into
        or remove elements from the stream (as may be the case for
        `flatMap` or `filter` operations).  Thus the count is the
        size of the `List` and there is no need to execute the pipeline
        and, as a side-effect, print out the list elements.
        """
        ...


    def anyMatch(self, predicate: "Predicate"["T"]) -> bool:
        """
        Returns whether any elements of this stream match the provided
        predicate.  May not evaluate the predicate on all elements if not
        necessary for determining the result.  If the stream is empty then
        `False` is returned and the predicate is not evaluated.
        
        This is a <a href="package-summary.html#StreamOps">short-circuiting
        terminal operation</a>.

        Arguments
        - predicate: a <a href="package-summary.html#NonInterference">non-interfering</a>,
                         <a href="package-summary.html#Statelessness">stateless</a>
                         predicate to apply to elements of this stream

        Returns
        - `True` if any elements of the stream match the provided
        predicate, otherwise `False`

        Unknown Tags
        - This method evaluates the *existential quantification* of the
        predicate over the elements of the stream (for some x P(x)).
        """
        ...


    def allMatch(self, predicate: "Predicate"["T"]) -> bool:
        """
        Returns whether all elements of this stream match the provided predicate.
        May not evaluate the predicate on all elements if not necessary for
        determining the result.  If the stream is empty then `True` is
        returned and the predicate is not evaluated.
        
        This is a <a href="package-summary.html#StreamOps">short-circuiting
        terminal operation</a>.

        Arguments
        - predicate: a <a href="package-summary.html#NonInterference">non-interfering</a>,
                         <a href="package-summary.html#Statelessness">stateless</a>
                         predicate to apply to elements of this stream

        Returns
        - `True` if either all elements of the stream match the
        provided predicate or the stream is empty, otherwise `False`

        Unknown Tags
        - This method evaluates the *universal quantification* of the
        predicate over the elements of the stream (for all x P(x)).  If the
        stream is empty, the quantification is said to be *vacuously
        satisfied* and is always `True` (regardless of P(x)).
        """
        ...


    def noneMatch(self, predicate: "Predicate"["T"]) -> bool:
        """
        Returns whether no elements of this stream match the provided predicate.
        May not evaluate the predicate on all elements if not necessary for
        determining the result.  If the stream is empty then `True` is
        returned and the predicate is not evaluated.
        
        This is a <a href="package-summary.html#StreamOps">short-circuiting
        terminal operation</a>.

        Arguments
        - predicate: a <a href="package-summary.html#NonInterference">non-interfering</a>,
                         <a href="package-summary.html#Statelessness">stateless</a>
                         predicate to apply to elements of this stream

        Returns
        - `True` if either no elements of the stream match the
        provided predicate or the stream is empty, otherwise `False`

        Unknown Tags
        - This method evaluates the *universal quantification* of the
        negated predicate over the elements of the stream (for all x ~P(x)).  If
        the stream is empty, the quantification is said to be vacuously satisfied
        and is always `True`, regardless of P(x).
        """
        ...


    def findFirst(self) -> "Optional"["T"]:
        """
        Returns an Optional describing the first element of this stream,
        or an empty `Optional` if the stream is empty.  If the stream has
        no encounter order, then any element may be returned.
        
        This is a <a href="package-summary.html#StreamOps">short-circuiting
        terminal operation</a>.

        Returns
        - an `Optional` describing the first element of this stream,
        or an empty `Optional` if the stream is empty

        Raises
        - NullPointerException: if the element selected is null
        """
        ...


    def findAny(self) -> "Optional"["T"]:
        """
        Returns an Optional describing some element of the stream, or an
        empty `Optional` if the stream is empty.
        
        This is a <a href="package-summary.html#StreamOps">short-circuiting
        terminal operation</a>.
        
        The behavior of this operation is explicitly nondeterministic; it is
        free to select any element in the stream.  This is to allow for maximal
        performance in parallel operations; the cost is that multiple invocations
        on the same source may not return the same result.  (If a stable result
        is desired, use .findFirst() instead.)

        Returns
        - an `Optional` describing some element of this stream, or an
        empty `Optional` if the stream is empty

        Raises
        - NullPointerException: if the element selected is null

        See
        - .findFirst()
        """
        ...


    @staticmethod
    def builder() -> "Builder"["T"]:
        """
        Returns a builder for a `Stream`.
        
        Type `<T>`: type of elements

        Returns
        - a stream builder
        """
        ...


    @staticmethod
    def empty() -> "Stream"["T"]:
        """
        Returns an empty sequential `Stream`.
        
        Type `<T>`: the type of stream elements

        Returns
        - an empty sequential stream
        """
        ...


    @staticmethod
    def of(t: "T") -> "Stream"["T"]:
        """
        Returns a sequential `Stream` containing a single element.
        
        Type `<T>`: the type of stream elements

        Arguments
        - t: the single element

        Returns
        - a singleton sequential stream
        """
        ...


    @staticmethod
    def ofNullable(t: "T") -> "Stream"["T"]:
        """
        Returns a sequential `Stream` containing a single element, if
        non-null, otherwise returns an empty `Stream`.
        
        Type `<T>`: the type of stream elements

        Arguments
        - t: the single element

        Returns
        - a stream with a single element if the specified element
                is non-null, otherwise an empty stream

        Since
        - 9
        """
        ...


    @staticmethod
    def of(*values: Tuple["T", ...]) -> "Stream"["T"]:
        """
        Returns a sequential ordered stream whose elements are the specified values.
        
        Type `<T>`: the type of stream elements

        Arguments
        - values: the elements of the new stream

        Returns
        - the new stream
        """
        ...


    @staticmethod
    def iterate(seed: "T", f: "UnaryOperator"["T"]) -> "Stream"["T"]:
        """
        Returns an infinite sequential ordered `Stream` produced by iterative
        application of a function `f` to an initial element `seed`,
        producing a `Stream` consisting of `seed`, `f(seed)`,
        `f(f(seed))`, etc.
        
        The first element (position `0`) in the `Stream` will be
        the provided `seed`.  For `n > 0`, the element at position
        `n`, will be the result of applying the function `f` to the
        element at position `n - 1`.
        
        The action of applying `f` for one element
        <a href="../concurrent/package-summary.html#MemoryVisibility">*happens-before*</a>
        the action of applying `f` for subsequent elements.  For any given
        element the action may be performed in whatever thread the library
        chooses.
        
        Type `<T>`: the type of stream elements

        Arguments
        - seed: the initial element
        - f: a function to be applied to the previous element to produce
                 a new element

        Returns
        - a new sequential `Stream`
        """
        ...


    @staticmethod
    def iterate(seed: "T", hasNext: "Predicate"["T"], next: "UnaryOperator"["T"]) -> "Stream"["T"]:
        """
        Returns a sequential ordered `Stream` produced by iterative
        application of the given `next` function to an initial element,
        conditioned on satisfying the given `hasNext` predicate.  The
        stream terminates as soon as the `hasNext` predicate returns False.
        
        `Stream.iterate` should produce the same sequence of elements as
        produced by the corresponding for-loop:
        ````for (T index=seed; hasNext.test(index); index = next.apply(index)) {
                ...`
        }```
        
        The resulting sequence may be empty if the `hasNext` predicate
        does not hold on the seed value.  Otherwise the first element will be the
        supplied `seed` value, the next element (if present) will be the
        result of applying the `next` function to the `seed` value,
        and so on iteratively until the `hasNext` predicate indicates that
        the stream should terminate.
        
        The action of applying the `hasNext` predicate to an element
        <a href="../concurrent/package-summary.html#MemoryVisibility">*happens-before*</a>
        the action of applying the `next` function to that element.  The
        action of applying the `next` function for one element
        *happens-before* the action of applying the `hasNext`
        predicate for subsequent elements.  For any given element an action may
        be performed in whatever thread the library chooses.
        
        Type `<T>`: the type of stream elements

        Arguments
        - seed: the initial element
        - hasNext: a predicate to apply to elements to determine when the
                       stream must terminate.
        - next: a function to be applied to the previous element to produce
                    a new element

        Returns
        - a new sequential `Stream`

        Since
        - 9
        """
        ...


    @staticmethod
    def generate(s: "Supplier"["T"]) -> "Stream"["T"]:
        """
        Returns an infinite sequential unordered stream where each element is
        generated by the provided `Supplier`.  This is suitable for
        generating constant streams, streams of random elements, etc.
        
        Type `<T>`: the type of stream elements

        Arguments
        - s: the `Supplier` of generated elements

        Returns
        - a new infinite sequential unordered `Stream`
        """
        ...


    @staticmethod
    def concat(a: "Stream"["T"], b: "Stream"["T"]) -> "Stream"["T"]:
        """
        Creates a lazily concatenated stream whose elements are all the
        elements of the first stream followed by all the elements of the
        second stream.  The resulting stream is ordered if both
        of the input streams are ordered, and parallel if either of the input
        streams is parallel.  When the resulting stream is closed, the close
        handlers for both input streams are invoked.
        
        This method operates on the two input streams and binds each stream
        to its source.  As a result subsequent modifications to an input stream
        source may not be reflected in the concatenated stream result.
        
        Type `<T>`: The type of stream elements

        Arguments
        - a: the first stream
        - b: the second stream

        Returns
        - the concatenation of the two input streams

        Unknown Tags
        - Use caution when constructing streams from repeated concatenation.
        Accessing an element of a deeply concatenated stream can result in deep
        call chains, or even `StackOverflowError`.
        
        Subsequent changes to the sequential/parallel execution mode of the
        returned stream are not guaranteed to be propagated to the input streams.
        - To preserve optimization opportunities this method binds each stream to
        its source and accepts only two streams as parameters.  For example, the
        exact size of the concatenated stream source can be computed if the exact
        size of each input stream source is known.
        To concatenate more streams without binding, or without nested calls to
        this method, try creating a stream of streams and flat-mapping with the
        identity function, for example:
        ````Stream<T> concat = Stream.of(s1, s2, s3, s4).flatMap(s -> s);````
        """
        ...


    class Builder(Consumer):
        """
        A mutable builder for a `Stream`.  This allows the creation of a
        `Stream` by generating elements individually and adding them to the
        `Builder` (without the copying overhead that comes from using
        an `ArrayList` as a temporary buffer.)
        
        A stream builder has a lifecycle, which starts in a building
        phase, during which elements can be added, and then transitions to a built
        phase, after which elements may not be added.  The built phase begins
        when the .build() method is called, which creates an ordered
        `Stream` whose elements are the elements that were added to the stream
        builder, in the order they were added.
        
        Type `<T>`: the type of stream elements

        See
        - Stream.builder()

        Since
        - 1.8
        """

        def accept(self, t: "T") -> None:
            """
            Adds an element to the stream being built.

            Raises
            - IllegalStateException: if the builder has already transitioned to
            the built state
            """
            ...


        def add(self, t: "T") -> "Builder"["T"]:
            """
            Adds an element to the stream being built.

            Arguments
            - t: the element to add

            Returns
            - `this` builder

            Raises
            - IllegalStateException: if the builder has already transitioned to
            the built state

            Unknown Tags
            - The default implementation behaves as if:
            ````accept(t)
                return this;````
            """
            ...


        def build(self) -> "Stream"["T"]:
            """
            Builds the stream, transitioning this builder to the built state.
            An `IllegalStateException` is thrown if there are further attempts
            to operate on the builder after it has entered the built state.

            Returns
            - the built stream

            Raises
            - IllegalStateException: if the builder has already transitioned to
            the built state
            """
            ...



"""
Python module generated from Java source file org.bukkit.metadata.MetadataStore

Java source file obtained from artifact spigot-api version 1.21.4-R0.1-20250303.102353-42

Because this Python module is automatically generated, it may contain errors
and/or code that cannot be parsed. Please report these issues at
https://github.com/magicmq/docs-translator/issues
"""
from org.bukkit.metadata import *
from org.bukkit.plugin import Plugin
from typing import Any, Callable, Iterable, Tuple


class MetadataStore:

    def setMetadata(self, subject: "T", metadataKey: str, newMetadataValue: "MetadataValue") -> None:
        """
        Adds a metadata value to an object.

        Arguments
        - subject: The object receiving the metadata.
        - metadataKey: A unique key to identify this metadata.
        - newMetadataValue: The metadata value to apply.

        Raises
        - IllegalArgumentException: If value is null, or the owning plugin
            is null
        """
        ...


    def getMetadata(self, subject: "T", metadataKey: str) -> list["MetadataValue"]:
        """
        Returns all metadata values attached to an object. If multiple plugins
        have attached metadata, each will value will be included.

        Arguments
        - subject: the object being interrogated.
        - metadataKey: the unique metadata key being sought.

        Returns
        - A list of values, one for each plugin that has set the
            requested value.
        """
        ...


    def hasMetadata(self, subject: "T", metadataKey: str) -> bool:
        """
        Tests to see if a metadata attribute has been set on an object.

        Arguments
        - subject: the object upon which the has-metadata test is
            performed.
        - metadataKey: the unique metadata key being queried.

        Returns
        - the existence of the metadataKey within subject.
        """
        ...


    def removeMetadata(self, subject: "T", metadataKey: str, owningPlugin: "Plugin") -> None:
        """
        Removes a metadata item owned by a plugin from a subject.

        Arguments
        - subject: the object to remove the metadata from.
        - metadataKey: the unique metadata key identifying the metadata to
            remove.
        - owningPlugin: the plugin attempting to remove a metadata item.

        Raises
        - IllegalArgumentException: If plugin is null
        """
        ...


    def invalidateAll(self, owningPlugin: "Plugin") -> None:
        """
        Invalidates all metadata in the metadata store that originates from the
        given plugin. Doing this will force each invalidated metadata item to
        be recalculated the next time it is accessed.

        Arguments
        - owningPlugin: the plugin requesting the invalidation.

        Raises
        - IllegalArgumentException: If plugin is null
        """
        ...



"""
Python module generated from Java source file com.google.common.graph.UndirectedMultiNetworkConnections

Java source file obtained from artifact guava version 33.3.1-jre

Because this Python module is automatically generated, it may contain errors
and/or code that cannot be parsed. Please report these issues at
https://github.com/magicmq/docs-translator/issues
"""
from com.google.common.collect import HashMultiset
from com.google.common.collect import ImmutableMap
from com.google.common.collect import Multiset
from com.google.common.graph import *
from com.google.errorprone.annotations.concurrent import LazyInit
from java.lang.ref import Reference
from java.lang.ref import SoftReference
from java.util import Collections
from javax.annotation import CheckForNull
from typing import Any, Callable, Iterable, Tuple


class UndirectedMultiNetworkConnections(AbstractUndirectedNetworkConnections):
    """
    An implementation of NetworkConnections for undirected networks with parallel edges.
    
    Type `<N>`: Node parameter type
    
    Type `<E>`: Edge parameter type

    Author(s)
    - James Sexton
    """

    def adjacentNodes(self) -> set["N"]:
        ...


    def edgesConnecting(self, node: "N") -> set["E"]:
        ...


    def removeInEdge(self, edge: "E", isSelfLoop: bool) -> "N":
        ...


    def removeOutEdge(self, edge: "E") -> "N":
        ...


    def addInEdge(self, edge: "E", node: "N", isSelfLoop: bool) -> None:
        ...


    def addOutEdge(self, edge: "E", node: "N") -> None:
        ...



"""
Python module generated from Java source file org.bukkit.material.Mushroom

Java source file obtained from artifact spigot-api version 1.21.4-R0.1-20250303.102353-42

Because this Python module is automatically generated, it may contain errors
and/or code that cannot be parsed. Please report these issues at
https://github.com/magicmq/docs-translator/issues
"""
from com.google.common.base import Preconditions
from java.util import EnumSet
from org.bukkit import Material
from org.bukkit.block import BlockFace
from org.bukkit.material import *
from org.bukkit.material.types import MushroomBlockTexture
from typing import Any, Callable, Iterable, Tuple


class Mushroom(MaterialData):
    """
    Represents a huge mushroom block with certain combinations of faces set to
    cap, pores or stem.

    See
    - Material.LEGACY_HUGE_MUSHROOM_2

    Deprecated
    - all usage of MaterialData is deprecated and subject to removal.
    Use org.bukkit.block.data.BlockData.
    """

    def __init__(self, shroom: "Material"):
        """
        Constructs a brown/red mushroom block with all sides set to pores.

        Arguments
        - shroom: A brown or red mushroom material type.

        See
        - Material.LEGACY_HUGE_MUSHROOM_2
        """
        ...


    def __init__(self, shroom: "Material", capFace: "BlockFace"):
        """
        Constructs a brown/red mushroom cap block with the specified face or
        faces set to cap texture.
        
        Setting any of the four sides will also set the top to cap.
        
        To set two side faces at once use e.g. north-west.
        
        Specify self to set all six faces at once.

        Arguments
        - shroom: A brown or red mushroom material type.
        - capFace: The face or faces to set to mushroom cap texture.

        See
        - BlockFace
        """
        ...


    def __init__(self, shroom: "Material", texture: "MushroomBlockTexture"):
        """
        Constructs a brown/red mushroom block with the specified textures.

        Arguments
        - shroom: A brown or red mushroom material type.
        - texture: The textured mushroom faces.

        See
        - Material.LEGACY_HUGE_MUSHROOM_2
        """
        ...


    def __init__(self, shroom: "Material", data: int):
        """
        Arguments
        - shroom: the type
        - data: the raw data value

        Deprecated
        - Magic value
        """
        ...


    def isStem(self) -> bool:
        """
        Returns
        - Whether this is a mushroom stem.
        """
        ...


    def setStem(self) -> None:
        """
        Sets this to be a mushroom stem.

        See
        - MushroomBlockTexture.ALL_STEM

        Deprecated
        - Use
        .setBlockTexture(org.bukkit.material.types.MushroomBlockTexture)
        with MushroomBlockTexture.STEM_SIDES or
        MushroomBlockTexture.ALL_STEM
        """
        ...


    def getBlockTexture(self) -> "MushroomBlockTexture":
        """
        Gets the mushroom texture of this block.

        Returns
        - The mushroom texture of this block
        """
        ...


    def setBlockTexture(self, texture: "MushroomBlockTexture") -> None:
        """
        Sets the mushroom texture of this block.

        Arguments
        - texture: The mushroom texture to set
        """
        ...


    def isFacePainted(self, face: "BlockFace") -> bool:
        """
        Checks whether a face of the block is painted with cap texture.

        Arguments
        - face: The face to check.

        Returns
        - True if it is painted.
        """
        ...


    def setFacePainted(self, face: "BlockFace", painted: bool) -> None:
        """
        Set a face of the block to be painted or not. Note that due to the
        nature of how the data is stored, setting a face painted or not is not
        guaranteed to leave the other faces unchanged.

        Arguments
        - face: The face to paint or unpaint.
        - painted: True if you want to paint it, False if you want the
            pores to show.

        Deprecated
        - Use MushroomBlockType cap options
        """
        ...


    def getPaintedFaces(self) -> set["BlockFace"]:
        """
        Returns
        - A set of all faces that are currently painted (an empty set if
            it is a stem)
        """
        ...


    def toString(self) -> str:
        ...


    def clone(self) -> "Mushroom":
        ...



"""
Python module generated from Java source file org.bukkit.inventory.PlayerInventory

Java source file obtained from artifact spigot-api version 1.21.4-R0.1-20250303.102353-42

Because this Python module is automatically generated, it may contain errors
and/or code that cannot be parsed. Please report these issues at
https://github.com/magicmq/docs-translator/issues
"""
from org.bukkit.entity import HumanEntity
from org.bukkit.inventory import *
from typing import Any, Callable, Iterable, Tuple


class PlayerInventory(Inventory):
    """
    Interface to the inventory of a Player, including the four armor slots and any extra slots.
    """

    def getArmorContents(self) -> list["ItemStack"]:
        """
        Gets all ItemStacks from the armor slots.

        Returns
        - all the ItemStacks from the armor slots. Individual items can be
        null and are returned in a fixed order starting from the boots and going
        up to the helmet
        """
        ...


    def getExtraContents(self) -> list["ItemStack"]:
        """
        Get all additional ItemStacks stored in this inventory.
        
        NB: What defines an extra slot is up to the implementation, however it
        will not be contained within .getStorageContents() or
        .getArmorContents()

        Returns
        - All additional ItemStacks. Individual items can be null.
        """
        ...


    def getHelmet(self) -> "ItemStack":
        """
        Return the ItemStack from the helmet slot

        Returns
        - The ItemStack in the helmet slot
        """
        ...


    def getChestplate(self) -> "ItemStack":
        """
        Return the ItemStack from the chestplate slot

        Returns
        - The ItemStack in the chestplate slot
        """
        ...


    def getLeggings(self) -> "ItemStack":
        """
        Return the ItemStack from the leg slot

        Returns
        - The ItemStack in the leg slot
        """
        ...


    def getBoots(self) -> "ItemStack":
        """
        Return the ItemStack from the boots slot

        Returns
        - The ItemStack in the boots slot
        """
        ...


    def setItem(self, index: int, item: "ItemStack") -> None:
        """
        Stores the ItemStack at the given index of the inventory.
        
        Indexes 0 through 8 refer to the hotbar. 9 through 35 refer to the main inventory, counting up from 9 at the top
        left corner of the inventory, moving to the right, and moving to the row below it back on the left side when it
        reaches the end of the row. It follows the same path in the inventory like you would read a book.
        
        Indexes 36 through 39 refer to the armor slots. Though you can set armor with this method using these indexes,
        you are encouraged to use the provided methods for those slots.
        
        Index 40 refers to the off hand (shield) item slot. Though you can set off hand with this method using this index,
        you are encouraged to use the provided method for this slot.
        
        If you attempt to use this method with an index less than 0 or greater than 40, an ArrayIndexOutOfBounds
        exception will be thrown.

        Arguments
        - index: The index where to put the ItemStack
        - item: The ItemStack to set

        Raises
        - ArrayIndexOutOfBoundsException: when index &lt; 0 || index &gt; 40

        See
        - .setItemInOffHand(ItemStack)
        """
        ...


    def setItem(self, slot: "EquipmentSlot", item: "ItemStack") -> None:
        """
        Stores the ItemStack at the given equipment slot in the inventory.

        Arguments
        - slot: the slot to put the ItemStack
        - item: the ItemStack to set

        See
        - .setItem(int, ItemStack)
        """
        ...


    def getItem(self, slot: "EquipmentSlot") -> "ItemStack":
        """
        Gets the ItemStack at the given equipment slot in the inventory.

        Arguments
        - slot: the slot to get the ItemStack

        Returns
        - the ItemStack in the given slot or null if there is not one
        """
        ...


    def setArmorContents(self, items: list["ItemStack"]) -> None:
        """
        Put the given ItemStacks into the armor slots

        Arguments
        - items: The ItemStacks to use as armour
        """
        ...


    def setExtraContents(self, items: list["ItemStack"]) -> None:
        """
        Put the given ItemStacks into the extra slots
        
        See .getExtraContents() for an explanation of extra slots.

        Arguments
        - items: The ItemStacks to use as extra
        """
        ...


    def setHelmet(self, helmet: "ItemStack") -> None:
        """
        Put the given ItemStack into the helmet slot. This does not check if
        the ItemStack is a helmet

        Arguments
        - helmet: The ItemStack to use as helmet
        """
        ...


    def setChestplate(self, chestplate: "ItemStack") -> None:
        """
        Put the given ItemStack into the chestplate slot. This does not check
        if the ItemStack is a chestplate

        Arguments
        - chestplate: The ItemStack to use as chestplate
        """
        ...


    def setLeggings(self, leggings: "ItemStack") -> None:
        """
        Put the given ItemStack into the leg slot. This does not check if the
        ItemStack is a pair of leggings

        Arguments
        - leggings: The ItemStack to use as leggings
        """
        ...


    def setBoots(self, boots: "ItemStack") -> None:
        """
        Put the given ItemStack into the boots slot. This does not check if the
        ItemStack is a boots

        Arguments
        - boots: The ItemStack to use as boots
        """
        ...


    def getItemInMainHand(self) -> "ItemStack":
        """
        Gets a copy of the item the player is currently holding
        in their main hand.

        Returns
        - the currently held item
        """
        ...


    def setItemInMainHand(self, item: "ItemStack") -> None:
        """
        Sets the item the player is holding in their main hand.

        Arguments
        - item: The item to put into the player's hand
        """
        ...


    def getItemInOffHand(self) -> "ItemStack":
        """
        Gets a copy of the item the player is currently holding
        in their off hand.

        Returns
        - the currently held item
        """
        ...


    def setItemInOffHand(self, item: "ItemStack") -> None:
        """
        Sets the item the player is holding in their off hand.

        Arguments
        - item: The item to put into the player's hand
        """
        ...


    def getItemInHand(self) -> "ItemStack":
        """
        Gets a copy of the item the player is currently holding

        Returns
        - the currently held item

        See
        - .getItemInOffHand()

        Deprecated
        - players can duel wield now use the methods for the
             specific hand instead
        """
        ...


    def setItemInHand(self, stack: "ItemStack") -> None:
        """
        Sets the item the player is holding

        Arguments
        - stack: The item to put into the player's hand

        See
        - .setItemInOffHand(ItemStack)

        Deprecated
        - players can duel wield now use the methods for the
             specific hand instead
        """
        ...


    def getHeldItemSlot(self) -> int:
        """
        Get the slot number of the currently held item

        Returns
        - Held item slot number
        """
        ...


    def setHeldItemSlot(self, slot: int) -> None:
        """
        Set the slot number of the currently held item.
        
        This validates whether the slot is between 0 and 8 inclusive.

        Arguments
        - slot: The new slot number

        Raises
        - IllegalArgumentException: Thrown if slot is not between 0 and 8
            inclusive
        """
        ...


    def getHolder(self) -> "HumanEntity":
        ...



"""
Python module generated from Java source file org.bukkit.block.Biome

Java source file obtained from artifact spigot-api version 1.21.4-R0.1-20250303.102353-42

Because this Python module is automatically generated, it may contain errors
and/or code that cannot be parsed. Please report these issues at
https://github.com/magicmq/docs-translator/issues
"""
from com.google.common.base import Preconditions
from com.google.common.collect import Lists
from java.util import Locale
from org.bukkit import Bukkit
from org.bukkit import FeatureFlag
from org.bukkit import Keyed
from org.bukkit import NamespacedKey
from org.bukkit import Registry
from org.bukkit.block import *
from org.bukkit.packs import DataPack
from org.bukkit.registry import RegistryAware
from org.bukkit.util import OldEnum
from typing import Any, Callable, Iterable, Tuple


class Biome(OldEnum, Keyed, RegistryAware):
    """
    Holds all accepted Biomes in the server.
    
    The Biomes listed in this interface are present in the default server
    or can be enabled via a FeatureFlag.
    There may be additional biomes present in the server, for example from a DataPack
    which can be accessed via Registry.BIOME.
    """

    OCEAN = getBiome("ocean")
    PLAINS = getBiome("plains")
    DESERT = getBiome("desert")
    WINDSWEPT_HILLS = getBiome("windswept_hills")
    FOREST = getBiome("forest")
    TAIGA = getBiome("taiga")
    SWAMP = getBiome("swamp")
    MANGROVE_SWAMP = getBiome("mangrove_swamp")
    RIVER = getBiome("river")
    NETHER_WASTES = getBiome("nether_wastes")
    THE_END = getBiome("the_end")
    FROZEN_OCEAN = getBiome("frozen_ocean")
    FROZEN_RIVER = getBiome("frozen_river")
    SNOWY_PLAINS = getBiome("snowy_plains")
    MUSHROOM_FIELDS = getBiome("mushroom_fields")
    BEACH = getBiome("beach")
    JUNGLE = getBiome("jungle")
    SPARSE_JUNGLE = getBiome("sparse_jungle")
    DEEP_OCEAN = getBiome("deep_ocean")
    STONY_SHORE = getBiome("stony_shore")
    SNOWY_BEACH = getBiome("snowy_beach")
    BIRCH_FOREST = getBiome("birch_forest")
    DARK_FOREST = getBiome("dark_forest")
    PALE_GARDEN = getBiome("pale_garden")
    SNOWY_TAIGA = getBiome("snowy_taiga")
    OLD_GROWTH_PINE_TAIGA = getBiome("old_growth_pine_taiga")
    WINDSWEPT_FOREST = getBiome("windswept_forest")
    SAVANNA = getBiome("savanna")
    SAVANNA_PLATEAU = getBiome("savanna_plateau")
    BADLANDS = getBiome("badlands")
    WOODED_BADLANDS = getBiome("wooded_badlands")
    SMALL_END_ISLANDS = getBiome("small_end_islands")
    END_MIDLANDS = getBiome("end_midlands")
    END_HIGHLANDS = getBiome("end_highlands")
    END_BARRENS = getBiome("end_barrens")
    WARM_OCEAN = getBiome("warm_ocean")
    LUKEWARM_OCEAN = getBiome("lukewarm_ocean")
    COLD_OCEAN = getBiome("cold_ocean")
    DEEP_LUKEWARM_OCEAN = getBiome("deep_lukewarm_ocean")
    DEEP_COLD_OCEAN = getBiome("deep_cold_ocean")
    DEEP_FROZEN_OCEAN = getBiome("deep_frozen_ocean")
    THE_VOID = getBiome("the_void")
    SUNFLOWER_PLAINS = getBiome("sunflower_plains")
    WINDSWEPT_GRAVELLY_HILLS = getBiome("windswept_gravelly_hills")
    FLOWER_FOREST = getBiome("flower_forest")
    ICE_SPIKES = getBiome("ice_spikes")
    OLD_GROWTH_BIRCH_FOREST = getBiome("old_growth_birch_forest")
    OLD_GROWTH_SPRUCE_TAIGA = getBiome("old_growth_spruce_taiga")
    WINDSWEPT_SAVANNA = getBiome("windswept_savanna")
    ERODED_BADLANDS = getBiome("eroded_badlands")
    BAMBOO_JUNGLE = getBiome("bamboo_jungle")
    SOUL_SAND_VALLEY = getBiome("soul_sand_valley")
    CRIMSON_FOREST = getBiome("crimson_forest")
    WARPED_FOREST = getBiome("warped_forest")
    BASALT_DELTAS = getBiome("basalt_deltas")
    DRIPSTONE_CAVES = getBiome("dripstone_caves")
    LUSH_CAVES = getBiome("lush_caves")
    DEEP_DARK = getBiome("deep_dark")
    MEADOW = getBiome("meadow")
    GROVE = getBiome("grove")
    SNOWY_SLOPES = getBiome("snowy_slopes")
    FROZEN_PEAKS = getBiome("frozen_peaks")
    JAGGED_PEAKS = getBiome("jagged_peaks")
    STONY_PEAKS = getBiome("stony_peaks")
    CHERRY_GROVE = getBiome("cherry_grove")
    CUSTOM = Bukkit.getUnsafe().getCustomBiome()
    """
    Represents a custom Biome

    Deprecated
    - Biome is no longer an enum, custom biomes will have their own biome instance.
    """


    @staticmethod
    def getBiome(key: str) -> "Biome":
        ...


    def getKey(self) -> "NamespacedKey":
        """
        See
        - .isRegistered()

        Deprecated
        - A key might not always be present, use .getKeyOrThrow() instead.
        """
        ...


    @staticmethod
    def valueOf(name: str) -> "Biome":
        """
        Arguments
        - name: of the biome.

        Returns
        - the biome with the given name.

        Deprecated
        - only for backwards compatibility, use Registry.get(NamespacedKey) instead.
        """
        ...


    @staticmethod
    def values() -> list["Biome"]:
        """
        Returns
        - an array of all known biomes.

        Deprecated
        - use Registry.iterator().
        """
        ...



"""
Python module generated from Java source file com.google.common.collect.ForwardingMultimap

Java source file obtained from artifact guava version 33.3.1-jre

Because this Python module is automatically generated, it may contain errors
and/or code that cannot be parsed. Please report these issues at
https://github.com/magicmq/docs-translator/issues
"""
from com.google.common.annotations import GwtCompatible
from com.google.common.collect import *
from com.google.errorprone.annotations import CanIgnoreReturnValue
from javax.annotation import CheckForNull
from org.checkerframework.checker.nullness.qual import Nullable
from typing import Any, Callable, Iterable, Tuple


class ForwardingMultimap(ForwardingObject, Multimap):
    """
    A multimap which forwards all its method calls to another multimap. Subclasses should override
    one or more methods to modify the behavior of the backing multimap as desired per the <a
    href="http://en.wikipedia.org/wiki/Decorator_pattern">decorator pattern</a>.
    
    **`default` method warning:** This class does *not* forward calls to `default` methods. Instead, it inherits their default implementations. When those implementations
    invoke methods, they invoke methods on the `ForwardingMultimap`.

    Author(s)
    - Robert Konigsberg

    Since
    - 2.0
    """

    def asMap(self) -> dict["K", Iterable["V"]]:
        ...


    def clear(self) -> None:
        ...


    def containsEntry(self, key: "Object", value: "Object") -> bool:
        ...


    def containsKey(self, key: "Object") -> bool:
        ...


    def containsValue(self, value: "Object") -> bool:
        ...


    def entries(self) -> Iterable["Entry"["K", "V"]]:
        ...


    def get(self, key: "K") -> Iterable["V"]:
        ...


    def isEmpty(self) -> bool:
        ...


    def keys(self) -> "Multiset"["K"]:
        ...


    def keySet(self) -> set["K"]:
        ...


    def put(self, key: "K", value: "V") -> bool:
        ...


    def putAll(self, key: "K", values: Iterable["V"]) -> bool:
        ...


    def putAll(self, multimap: "Multimap"["K", "V"]) -> bool:
        ...


    def remove(self, key: "Object", value: "Object") -> bool:
        ...


    def removeAll(self, key: "Object") -> Iterable["V"]:
        ...


    def replaceValues(self, key: "K", values: Iterable["V"]) -> Iterable["V"]:
        ...


    def size(self) -> int:
        ...


    def values(self) -> Iterable["V"]:
        ...


    def equals(self, object: "Object") -> bool:
        ...


    def hashCode(self) -> int:
        ...



"""
Python module generated from Java source file org.bukkit.inventory.DoubleChestInventory

Java source file obtained from artifact spigot-api version 1.21.4-R0.1-20250303.102353-42

Because this Python module is automatically generated, it may contain errors
and/or code that cannot be parsed. Please report these issues at
https://github.com/magicmq/docs-translator/issues
"""
from org.bukkit.block import DoubleChest
from org.bukkit.inventory import *
from typing import Any, Callable, Iterable, Tuple


class DoubleChestInventory(Inventory):
    """
    Interface to the inventory of a Double Chest.
    """

    def getLeftSide(self) -> "Inventory":
        """
        Get the left half of this double chest.

        Returns
        - The left side inventory
        """
        ...


    def getRightSide(self) -> "Inventory":
        """
        Get the right side of this double chest.

        Returns
        - The right side inventory
        """
        ...


    def getHolder(self) -> "DoubleChest":
        ...



"""
Python module generated from Java source file com.google.common.collect.ImmutableClassToInstanceMap

Java source file obtained from artifact guava version 33.3.1-jre

Because this Python module is automatically generated, it may contain errors
and/or code that cannot be parsed. Please report these issues at
https://github.com/magicmq/docs-translator/issues
"""
from com.google.common.annotations import GwtIncompatible
from com.google.common.collect import *
from com.google.common.primitives import Primitives
from com.google.errorprone.annotations import CanIgnoreReturnValue
from com.google.errorprone.annotations import DoNotCall
from com.google.errorprone.annotations import Immutable
from java.io import Serializable
from javax.annotation import CheckForNull
from org.checkerframework.checker.nullness.qual import NonNull
from typing import Any, Callable, Iterable, Tuple


class ImmutableClassToInstanceMap(ForwardingMap, ClassToInstanceMap, Serializable):
    """
    A ClassToInstanceMap whose contents will never change, with many other important
    properties detailed at ImmutableCollection.

    Author(s)
    - Kevin Bourrillion

    Since
    - 2.0
    """

    @staticmethod
    def of() -> "ImmutableClassToInstanceMap"["B"]:
        """
        Returns an empty `ImmutableClassToInstanceMap`.
        
        **Performance note:** the instance returned is a singleton.

        Since
        - 19.0
        """
        ...


    @staticmethod
    def of(type: type["T"], value: "T") -> "ImmutableClassToInstanceMap"["B"]:
        """
        Returns an `ImmutableClassToInstanceMap` containing a single entry.

        Since
        - 19.0
        """
        ...


    @staticmethod
    def builder() -> "Builder"["B"]:
        """
        Returns a new builder. The generated builder is equivalent to the builder created by the Builder constructor.
        """
        ...


    @staticmethod
    def copyOf(map: dict[type["S"], "S"]) -> "ImmutableClassToInstanceMap"["B"]:
        """
        Returns an immutable map containing the same entries as `map`. If `map` somehow
        contains entries with duplicate keys (for example, if it is a `SortedMap` whose
        comparator is not *consistent with equals*), the results of this method are undefined.
        
        **Note:** Despite what the method name suggests, if `map` is an `ImmutableClassToInstanceMap`, no copy will actually be performed.

        Raises
        - NullPointerException: if any key or value in `map` is null
        - ClassCastException: if any value is not an instance of the type specified by its key
        """
        ...


    def getInstance(self, type: type["T"]) -> "T":
        ...


    def putInstance(self, type: type["T"], value: "T") -> "T":
        """
        Guaranteed to throw an exception and leave the map unmodified.

        Raises
        - UnsupportedOperationException: always

        Deprecated
        - Unsupported operation.
        """
        ...


    class Builder:
        """
        A builder for creating immutable class-to-instance maps. Example:
        
        ````static final ImmutableClassToInstanceMap<Handler> HANDLERS =
            new ImmutableClassToInstanceMap.Builder<Handler>()
                .put(FooHandler.class, new FooHandler())
                .put(BarHandler.class, new SubBarHandler())
                .put(Handler.class, new QuuxHandler())
                .build();````
        
        After invoking .build() it is still possible to add more entries and build again.
        Thus each map generated by this builder will be a superset of any map generated before it.

        Since
        - 2.0
        """

        def put(self, key: type["T"], value: "T") -> "Builder"["B"]:
            """
            Associates `key` with `value` in the built map. Duplicate keys are not allowed,
            and will cause .build to fail.
            """
            ...


        def putAll(self, map: dict[type["T"], "T"]) -> "Builder"["B"]:
            """
            Associates all of `map's` keys and values in the built map. Duplicate keys are not
            allowed, and will cause .build to fail.

            Raises
            - NullPointerException: if any key or value in `map` is null
            - ClassCastException: if any value is not an instance of the type specified by its key
            """
            ...


        def build(self) -> "ImmutableClassToInstanceMap"["B"]:
            """
            Returns a new immutable class-to-instance map containing the entries provided to this
            builder.

            Raises
            - IllegalArgumentException: if duplicate keys were added
            """
            ...



"""
Python module generated from Java source file java.nio.file.attribute.FileAttribute

Java source file obtained from artifact jdk version jdk

Because this Python module is automatically generated, it may contain errors
and/or code that cannot be parsed. Please report these issues at
https://github.com/magicmq/docs-translator/issues
"""
from java.nio.file.attribute import *
from typing import Any, Callable, Iterable, Tuple


class FileAttribute:

    def name(self) -> str:
        """
        Returns the attribute name.

        Returns
        - The attribute name
        """
        ...


    def value(self) -> "T":
        """
        Returns the attribute value.

        Returns
        - The attribute value
        """
        ...



"""
Python module generated from Java source file org.bukkit.block.SculkShrieker

Java source file obtained from artifact spigot-api version 1.21.4-R0.1-20250303.102353-42

Because this Python module is automatically generated, it may contain errors
and/or code that cannot be parsed. Please report these issues at
https://github.com/magicmq/docs-translator/issues
"""
from org.bukkit.block import *
from org.bukkit.entity import Player
from typing import Any, Callable, Iterable, Tuple


class SculkShrieker(TileState):
    """
    Represents a captured state of a sculk shrieker.
    """

    def getWarningLevel(self) -> int:
        """
        Gets the most recent warning level of this block.
        
        When the warning level reaches 4, the shrieker will attempt to spawn a
        Warden.

        Returns
        - current warning level
        """
        ...


    def setWarningLevel(self, level: int) -> None:
        """
        Sets the most recent warning level of this block.
        
        When the warning level reaches 4, the shrieker will attempt to spawn a
        Warden.

        Arguments
        - level: new warning level
        """
        ...


    def tryShriek(self, player: "Player") -> None:
        """
        Simulates a player causing a vibration.

        Arguments
        - player: the player that "caused" the shriek
        """
        ...



from .RaidTriggerEvent import RaidTriggerEvent
from .RaidStopEvent import RaidStopEvent
from .RaidFinishEvent import RaidFinishEvent
from .RaidSpawnWaveEvent import RaidSpawnWaveEvent
from .RaidEvent import RaidEvent


"""
Python module generated from Java source file com.google.common.io.LineReader

Java source file obtained from artifact guava version 33.3.1-jre

Because this Python module is automatically generated, it may contain errors
and/or code that cannot be parsed. Please report these issues at
https://github.com/magicmq/docs-translator/issues
"""
from com.google.common.annotations import GwtIncompatible
from com.google.common.annotations import J2ktIncompatible
from com.google.common.io import *
from com.google.errorprone.annotations import CanIgnoreReturnValue
from java.io import IOException
from java.io import Reader
from java.util import ArrayDeque
from java.util import Queue
from javax.annotation import CheckForNull
from typing import Any, Callable, Iterable, Tuple


class LineReader:
    """
    A class for reading lines of text. Provides the same functionality as java.io.BufferedReader.readLine() but for all Readable objects, not just instances of
    Reader.

    Author(s)
    - Chris Nokleberg

    Since
    - 1.0
    """

    def __init__(self, readable: "Readable"):
        """
        Creates a new instance that will read lines from the given `Readable` object.
        """
        ...


    def readLine(self) -> str:
        """
        Reads a line of text. A line is considered to be terminated by any one of a line feed (`'\n'`), a carriage return (`'\r'`), or a carriage return followed immediately by a
        linefeed (`"\r\n"`).

        Returns
        - a `String` containing the contents of the line, not including any
            line-termination characters, or `null` if the end of the stream has been reached.

        Raises
        - IOException: if an I/O error occurs
        """
        ...



"""
Python module generated from Java source file java.security.CodeSource

Java source file obtained from artifact jdk version jdk

Because this Python module is automatically generated, it may contain errors
and/or code that cannot be parsed. Please report these issues at
https://github.com/magicmq/docs-translator/issues
"""
from java.io import ByteArrayInputStream
from java.io import IOException
from java.net import SocketPermission
from java.net import URL
from java.security import *
from java.security.cert import *
from java.util import Hashtable
from java.util import Objects
from sun.net.util import URLUtil
from sun.security.util import IOUtils
from typing import Any, Callable, Iterable, Tuple


class CodeSource(Serializable):

    def __init__(self, url: "URL", certs: list["java.security.cert.Certificate"]):
        """
        Constructs a CodeSource and associates it with the specified
        location and set of certificates.

        Arguments
        - url: the location (URL).  It may be `null`.
        - certs: the certificate(s). It may be `null`. The contents
        of the array are copied to protect against subsequent modification.
        """
        ...


    def __init__(self, url: "URL", signers: list["CodeSigner"]):
        """
        Constructs a CodeSource and associates it with the specified
        location and set of code signers.

        Arguments
        - url: the location (URL).  It may be `null`.
        - signers: the code signers. It may be `null`. The contents
        of the array are copied to protect against subsequent modification.

        Since
        - 1.5
        """
        ...


    def hashCode(self) -> int:
        """
        Returns the hash code value for this object.

        Returns
        - a hash code value for this object.
        """
        ...


    def equals(self, obj: "Object") -> bool:
        """
        Tests for equality between the specified object and this
        object. Two CodeSource objects are considered equal if their
        locations are of identical value and if their signer certificate
        chains are of identical value. It is not required that
        the certificate chains be in the same order.

        Arguments
        - obj: the object to test for equality with this object.

        Returns
        - True if the objects are considered equal, False otherwise.
        """
        ...


    def getLocation(self) -> "URL":
        """
        Returns the location associated with this CodeSource.

        Returns
        - the location (URL), or `null` if no URL was supplied
        during construction.
        """
        ...


    def getCertificates(self) -> list["java.security.cert.Certificate"]:
        """
        Returns the certificates associated with this CodeSource.
        
        If this CodeSource object was created using the
        .CodeSource(URL url, CodeSigner[] signers)
        constructor then its certificate chains are extracted and used to
        create an array of Certificate objects. Each signer certificate is
        followed by its supporting certificate chain (which may be empty).
        Each signer certificate and its supporting certificate chain is ordered
        bottom-to-top (i.e., with the signer certificate first and the (root)
        certificate authority last).

        Returns
        - a copy of the certificate array, or `null` if there
        is none.
        """
        ...


    def getCodeSigners(self) -> list["CodeSigner"]:
        """
        Returns the code signers associated with this CodeSource.
        
        If this CodeSource object was created using the
        .CodeSource(URL url, java.security.cert.Certificate[] certs)
        constructor then its certificate chains are extracted and used to
        create an array of CodeSigner objects. Note that only X.509 certificates
        are examined - all other certificate types are ignored.

        Returns
        - a copy of the code signer array, or `null` if there
        is none.

        Since
        - 1.5
        """
        ...


    def implies(self, codesource: "CodeSource") -> bool:
        """
        Returns True if this CodeSource object "implies" the specified CodeSource.
        
        More specifically, this method makes the following checks.
        If any fail, it returns False. If they all succeed, it returns True.
        
        -  *codesource* must not be null.
        -  If this object's certificates are not null, then all
        of this object's certificates must be present in *codesource*'s
        certificates.
        -  If this object's location (getLocation()) is not null, then the
        following checks are made against this object's location and
        *codesource*'s:
          
            -   *codesource*'s location must not be null.
        
            -   If this object's location
                  equals *codesource*'s location, then return True.
        
            -   This object's protocol (getLocation().getProtocol()) must be
                  equal to *codesource*'s protocol, ignoring case.
        
            -   If this object's host (getLocation().getHost()) is not null,
                  then the SocketPermission
                  constructed with this object's host must imply the
                  SocketPermission constructed with *codesource*'s host.
        
            -   If this object's port (getLocation().getPort()) is not
                  equal to -1 (that is, if a port is specified), it must equal
                  *codesource*'s port or default port
                  (codesource.getLocation().getDefaultPort()).
        
            -   If this object's file (getLocation().getFile()) doesn't equal
                  *codesource*'s file, then the following checks are made:
                  If this object's file ends with "/-",
                  then *codesource*'s file must start with this object's
                  file (exclusive the trailing "-").
                  If this object's file ends with a "/*",
                  then *codesource*'s file must start with this object's
                  file and must not have any further "/" separators.
                  If this object's file doesn't end with a "/",
                  then *codesource*'s file must match this object's
                  file with a '/' appended.
        
            -   If this object's reference (getLocation().getRef()) is
                  not null, it must equal *codesource*'s reference.
        
          
        
        
        For example, the codesource objects with the following locations
        and null certificates all imply
        the codesource with the location "http://www.example.com/classes/foo.jar"
        and null certificates:
        ```
            http:
            http://*.example.com/classes/*
            http://www.example.com/classes/-
            http://www.example.com/classes/foo.jar
        ```
        
        Note that if this CodeSource has a null location and a null
        certificate chain, then it implies every other CodeSource.

        Arguments
        - codesource: CodeSource to compare against.

        Returns
        - True if the specified codesource is implied by this codesource,
        False if not.
        """
        ...


    def toString(self) -> str:
        """
        Returns a string describing this CodeSource, telling its
        URL and certificates.

        Returns
        - information about this CodeSource.
        """
        ...



"""
Python module generated from Java source file java.util.concurrent.ConcurrentNavigableMap

Java source file obtained from artifact jdk version jdk

Because this Python module is automatically generated, it may contain errors
and/or code that cannot be parsed. Please report these issues at
https://github.com/magicmq/docs-translator/issues
"""
from java.util import NavigableMap
from java.util import NavigableSet
from java.util.concurrent import *
from typing import Any, Callable, Iterable, Tuple


class ConcurrentNavigableMap(ConcurrentMap, NavigableMap):
    """
    A ConcurrentMap supporting NavigableMap operations,
    and recursively so for its navigable sub-maps.
    
    This interface is a member of the
    <a href="/java.base/java/util/package-summary.html#CollectionsFramework">
    Java Collections Framework</a>.
    
    Type `<K>`: the type of keys maintained by this map
    
    Type `<V>`: the type of mapped values

    Author(s)
    - Doug Lea

    Since
    - 1.6
    """

    def subMap(self, fromKey: "K", fromInclusive: bool, toKey: "K", toInclusive: bool) -> "ConcurrentNavigableMap"["K", "V"]:
        """
        Raises
        - ClassCastException: 
        - NullPointerException: 
        - IllegalArgumentException: 
        """
        ...


    def headMap(self, toKey: "K", inclusive: bool) -> "ConcurrentNavigableMap"["K", "V"]:
        """
        Raises
        - ClassCastException: 
        - NullPointerException: 
        - IllegalArgumentException: 
        """
        ...


    def tailMap(self, fromKey: "K", inclusive: bool) -> "ConcurrentNavigableMap"["K", "V"]:
        """
        Raises
        - ClassCastException: 
        - NullPointerException: 
        - IllegalArgumentException: 
        """
        ...


    def subMap(self, fromKey: "K", toKey: "K") -> "ConcurrentNavigableMap"["K", "V"]:
        """
        Raises
        - ClassCastException: 
        - NullPointerException: 
        - IllegalArgumentException: 
        """
        ...


    def headMap(self, toKey: "K") -> "ConcurrentNavigableMap"["K", "V"]:
        """
        Raises
        - ClassCastException: 
        - NullPointerException: 
        - IllegalArgumentException: 
        """
        ...


    def tailMap(self, fromKey: "K") -> "ConcurrentNavigableMap"["K", "V"]:
        """
        Raises
        - ClassCastException: 
        - NullPointerException: 
        - IllegalArgumentException: 
        """
        ...


    def descendingMap(self) -> "ConcurrentNavigableMap"["K", "V"]:
        """
        Returns a reverse order view of the mappings contained in this map.
        The descending map is backed by this map, so changes to the map are
        reflected in the descending map, and vice-versa.
        
        The returned map has an ordering equivalent to
        java.util.Collections.reverseOrder(Comparator) Collections.reverseOrder`(comparator())`.
        The expression `m.descendingMap().descendingMap()` returns a
        view of `m` essentially equivalent to `m`.

        Returns
        - a reverse order view of this map
        """
        ...


    def navigableKeySet(self) -> "NavigableSet"["K"]:
        """
        Returns a NavigableSet view of the keys contained in this map.
        The set's iterator returns the keys in ascending order.
        The set is backed by the map, so changes to the map are
        reflected in the set, and vice-versa.  The set supports element
        removal, which removes the corresponding mapping from the map,
        via the `Iterator.remove`, `Set.remove`,
        `removeAll`, `retainAll`, and `clear`
        operations.  It does not support the `add` or `addAll`
        operations.
        
        The view's iterators and spliterators are
        <a href="package-summary.html#Weakly">*weakly consistent*</a>.

        Returns
        - a navigable set view of the keys in this map
        """
        ...


    def keySet(self) -> "NavigableSet"["K"]:
        """
        Returns a NavigableSet view of the keys contained in this map.
        The set's iterator returns the keys in ascending order.
        The set is backed by the map, so changes to the map are
        reflected in the set, and vice-versa.  The set supports element
        removal, which removes the corresponding mapping from the map,
        via the `Iterator.remove`, `Set.remove`,
        `removeAll`, `retainAll`, and `clear`
        operations.  It does not support the `add` or `addAll`
        operations.
        
        The view's iterators and spliterators are
        <a href="package-summary.html#Weakly">*weakly consistent*</a>.
        
        This method is equivalent to method `navigableKeySet`.

        Returns
        - a navigable set view of the keys in this map
        """
        ...


    def descendingKeySet(self) -> "NavigableSet"["K"]:
        """
        Returns a reverse order NavigableSet view of the keys contained in this map.
        The set's iterator returns the keys in descending order.
        The set is backed by the map, so changes to the map are
        reflected in the set, and vice-versa.  The set supports element
        removal, which removes the corresponding mapping from the map,
        via the `Iterator.remove`, `Set.remove`,
        `removeAll`, `retainAll`, and `clear`
        operations.  It does not support the `add` or `addAll`
        operations.
        
        The view's iterators and spliterators are
        <a href="package-summary.html#Weakly">*weakly consistent*</a>.

        Returns
        - a reverse order navigable set view of the keys in this map
        """
        ...



"""
Python module generated from Java source file com.google.common.collect.RegularImmutableList

Java source file obtained from artifact guava version 33.3.1-jre

Because this Python module is automatically generated, it may contain errors
and/or code that cannot be parsed. Please report these issues at
https://github.com/magicmq/docs-translator/issues
"""
from com.google.common.annotations import GwtCompatible
from com.google.common.annotations import GwtIncompatible
from com.google.common.annotations import J2ktIncompatible
from com.google.common.annotations import VisibleForTesting
from com.google.common.collect import *
from java.util import Spliterator
from org.checkerframework.checker.nullness.qual import Nullable
from typing import Any, Callable, Iterable, Tuple


class RegularImmutableList(ImmutableList):
    """
    Implementation of ImmutableList backed by a simple array.

    Author(s)
    - Kevin Bourrillion
    """

    def size(self) -> int:
        ...


    def get(self, index: int) -> "E":
        ...


    def listIterator(self, index: int) -> "UnmodifiableListIterator"["E"]:
        ...


    def spliterator(self) -> "Spliterator"["E"]:
        ...



"""
Python module generated from Java source file org.bukkit.generator.ChunkGenerator

Java source file obtained from artifact spigot-api version 1.21.4-R0.1-20250303.102353-42

Because this Python module is automatically generated, it may contain errors
and/or code that cannot be parsed. Please report these issues at
https://github.com/magicmq/docs-translator/issues
"""
from java.util import Random
from org.bukkit import Bukkit
from org.bukkit import HeightMap
from org.bukkit import Location
from org.bukkit import Material
from org.bukkit import World
from org.bukkit.block import Biome
from org.bukkit.block import Block
from org.bukkit.block.data import BlockData
from org.bukkit.generator import *
from org.bukkit.material import MaterialData
from typing import Any, Callable, Iterable, Tuple


class ChunkGenerator:
    """
    A chunk generator is responsible for the initial shaping of an entire
    chunk. For example, the nether chunk generator should shape netherrack and
    soulsand.
    
    A chunk is generated in multiple steps, those steps are always in the same
    order. Between those steps however an unlimited time may pass. This means, a
    chunk may generated until the surface step and continue with the bedrock step
    after one or multiple server restarts or even after multiple Minecraft
    versions.
    
    The order of generation is as follows
    <ol>
    - .generateNoise(WorldInfo, Random, int, int, ChunkData)
    - .generateSurface(WorldInfo, Random, int, int, ChunkData)
    - .generateBedrock(WorldInfo, Random, int, int, ChunkData)
    - .generateCaves(WorldInfo, Random, int, int, ChunkData)
    </ol>
    
    Every method listed above as well as
    .getBaseHeight(WorldInfo, Random, int, int, HeightMap)
    **must** be completely thread safe and able to handle multiple concurrent
    callers.
    
    Some aspects of world generation can be delegated to the Vanilla generator.
    The following methods can be overridden to enable this:
    
    - ChunkGenerator.shouldGenerateNoise() or ChunkGenerator.shouldGenerateNoise(WorldInfo, Random, int, int)
    - ChunkGenerator.shouldGenerateSurface() or ChunkGenerator.shouldGenerateSurface(WorldInfo, Random, int, int)
    - ChunkGenerator.shouldGenerateCaves() or ChunkGenerator.shouldGenerateCaves(WorldInfo, Random, int, int)
    - ChunkGenerator.shouldGenerateDecorations() or ChunkGenerator.shouldGenerateDecorations(WorldInfo, Random, int, int)
    - ChunkGenerator.shouldGenerateMobs() or ChunkGenerator.shouldGenerateMobs(WorldInfo, Random, int, int)
    - ChunkGenerator.shouldGenerateStructures() or ChunkGenerator.shouldGenerateStructures(WorldInfo, Random, int, int)
    """

    def generateNoise(self, worldInfo: "WorldInfo", random: "Random", chunkX: int, chunkZ: int, chunkData: "ChunkData") -> None:
        """
        Shapes the Chunk noise for the given coordinates.
        
        Notes:
        
        This method should **never** attempt to get the Chunk at the passed
        coordinates, as doing so may cause an infinite loop.
        
        This method should **never** modify the ChunkData at a later
        point of time.
        
        The Y-coordinate range should **never** be hardcoded, to get the
        Y-coordinate range use the methods ChunkData.getMinHeight() and
        ChunkData.getMaxHeight().
        
        If .shouldGenerateNoise() is set to True, the given
        ChunkData contains already the Vanilla noise generation.

        Arguments
        - worldInfo: The world info of the world this chunk will be used for
        - random: The random generator to use
        - chunkX: The X-coordinate of the chunk
        - chunkZ: The Z-coordinate of the chunk
        - chunkData: To modify
        """
        ...


    def generateSurface(self, worldInfo: "WorldInfo", random: "Random", chunkX: int, chunkZ: int, chunkData: "ChunkData") -> None:
        """
        Shapes the Chunk surface for the given coordinates.
        
        Notes:
        
        This method should **never** attempt to get the Chunk at the passed
        coordinates, as doing so may cause an infinite loop.
        
        This method should **never** modify the ChunkData at a later
        point of time.
        
        The Y-coordinate range should **never** be hardcoded, to get the
        Y-coordinate range use the methods ChunkData.getMinHeight() and
        ChunkData.getMaxHeight().
        
        If .shouldGenerateSurface() is set to True, the given
        ChunkData contains already the Vanilla surface generation.

        Arguments
        - worldInfo: The world info of the world this chunk will be used for
        - random: The random generator to use
        - chunkX: The X-coordinate of the chunk
        - chunkZ: The Z-coordinate of the chunk
        - chunkData: To modify
        """
        ...


    def generateBedrock(self, worldInfo: "WorldInfo", random: "Random", chunkX: int, chunkZ: int, chunkData: "ChunkData") -> None:
        """
        Shapes the Chunk bedrock layer for the given coordinates.
        
        Notes:
        
        This method should **never** attempt to get the Chunk at the passed
        coordinates, as doing so may cause an infinite loop.
        
        This method should **never** modify the ChunkData at a later
        point of time.
        
        The Y-coordinate range should **never** be hardcoded, to get the
        Y-coordinate range use the methods ChunkData.getMinHeight() and
        ChunkData.getMaxHeight().

        Arguments
        - worldInfo: The world info of the world this chunk will be used for
        - random: The random generator to use
        - chunkX: The X-coordinate of the chunk
        - chunkZ: The Z-coordinate of the chunk
        - chunkData: To modify
        """
        ...


    def generateCaves(self, worldInfo: "WorldInfo", random: "Random", chunkX: int, chunkZ: int, chunkData: "ChunkData") -> None:
        """
        Shapes the Chunk caves for the given coordinates.
        
        Notes:
        
        This method should **never** attempt to get the Chunk at the passed
        coordinates, as doing so may cause an infinite loop.
        
        This method should **never** modify the ChunkData at a later
        point of time.
        
        The Y-coordinate range should **never** be hardcoded, to get the
        Y-coordinate range use the methods ChunkData.getMinHeight() and
        ChunkData.getMaxHeight().
        
        If .shouldGenerateCaves() is set to True, the given
        ChunkData contains already the Vanilla cave generation.

        Arguments
        - worldInfo: The world info of the world this chunk will be used for
        - random: The random generator to use
        - chunkX: The X-coordinate of the chunk
        - chunkZ: The Z-coordinate of the chunk
        - chunkData: To modify
        """
        ...


    def getDefaultBiomeProvider(self, worldInfo: "WorldInfo") -> "BiomeProvider":
        """
        Gets called when no BiomeProvider is set in
        org.bukkit.WorldCreator or via the server configuration files. It
        is therefore possible that one plugin can provide the Biomes and another
        one the generation.
        
        Notes:
        
        If `null` is returned, than Vanilla biomes are used.
        
        This method only gets called once when the world is loaded. Returning
        another BiomeProvider later one is not respected.

        Arguments
        - worldInfo: The world info of the world the biome provider will be
        used for

        Returns
        - BiomeProvider to use to fill the biomes of a chunk
        """
        ...


    def getBaseHeight(self, worldInfo: "WorldInfo", random: "Random", x: int, z: int, heightMap: "HeightMap") -> int:
        """
        This method is similar to
        World.getHighestBlockAt(int, int, HeightMap). With the difference
        being, that the highest y coordinate should be the block before any
        surface, bedrock, caves or decoration is applied. Or in other words the
        highest block when only the noise is present at the chunk.
        
        Notes:
        
        When this method is not overridden, the Vanilla base height is used.
        
        This method should **never** attempt to get the Chunk at the passed
        coordinates, or use the method
        World.getHighestBlockAt(int, int, HeightMap), as doing so may
        cause an infinite loop.

        Arguments
        - worldInfo: The world info of the world this chunk will be used for
        - random: The random generator to use
        - x: The X-coordinate from world origin
        - z: The Z-coordinate from world origin
        - heightMap: From the highest block should be get

        Returns
        - The y coordinate of the highest block at the given location
        """
        ...


    def generateChunkData(self, world: "World", random: "Random", x: int, z: int, biome: "BiomeGrid") -> "ChunkData":
        """
        Shapes the chunk for the given coordinates.
        
        This method must return a ChunkData.
        
        Notes:
        
        This method should **never** attempt to get the Chunk at
        the passed coordinates, as doing so may cause an infinite loop
        
        This method should **never** modify a ChunkData after it has
        been returned.
        
        This method **must** return a ChunkData returned by ChunkGenerator.createChunkData(org.bukkit.World)

        Arguments
        - world: The world this chunk will be used for
        - random: The random generator to use
        - x: The X-coordinate of the chunk
        - z: The Z-coordinate of the chunk
        - biome: Proposed biome values for chunk - can be updated by
            generator

        Returns
        - ChunkData containing the types for each block created by this
            generator

        Deprecated
        - The generation is now split up and the new methods should be used, see ChunkGenerator
        """
        ...


    def canSpawn(self, world: "World", x: int, z: int) -> bool:
        """
        Tests if the specified location is valid for a natural spawn position

        Arguments
        - world: The world we're testing on
        - x: X-coordinate of the block to test
        - z: Z-coordinate of the block to test

        Returns
        - True if the location is valid, otherwise False
        """
        ...


    def getDefaultPopulators(self, world: "World") -> list["BlockPopulator"]:
        """
        Gets a list of default BlockPopulators to apply to a given
        world

        Arguments
        - world: World to apply to

        Returns
        - List containing any amount of BlockPopulators
        """
        ...


    def getFixedSpawnLocation(self, world: "World", random: "Random") -> "Location":
        """
        Gets a fixed spawn location to use for a given world.
        
        A null value is returned if a world should not use a fixed spawn point,
        and will instead attempt to find one randomly.

        Arguments
        - world: The world to locate a spawn point for
        - random: Random generator to use in the calculation

        Returns
        - Location containing a new spawn point, otherwise null
        """
        ...


    def isParallelCapable(self) -> bool:
        """
        Gets if this ChunkGenerator is parallel capable.
        
        See ChunkGenerator for more information.

        Returns
        - parallel capable status

        Deprecated
        - the chunk generation code should be thread safe
        """
        ...


    def shouldGenerateNoise(self) -> bool:
        """
        Gets if the server should generate Vanilla noise.
        
        The Vanilla noise is generated **before**
        .generateNoise(WorldInfo, Random, int, int, ChunkData) is called.
        
        This is method is not called (and has therefore no effect), if
        .shouldGenerateNoise(WorldInfo, Random, int, int) is overridden.

        Returns
        - True if the server should generate Vanilla noise

        See
        - .shouldGenerateNoise(WorldInfo, Random, int, int)
        """
        ...


    def shouldGenerateNoise(self, worldInfo: "WorldInfo", random: "Random", chunkX: int, chunkZ: int) -> bool:
        """
        Gets if the server should generate Vanilla noise.
        
        The Vanilla noise is generated **before**
        .generateNoise(WorldInfo, Random, int, int, ChunkData) is called.
        
        Only this method is called if both this and
        .shouldGenerateNoise() are overridden.

        Arguments
        - worldInfo: The world info of the world this chunk will be used for
        - random: The random generator to use
        - chunkX: The X-coordinate of the chunk
        - chunkZ: The Z-coordinate of the chunk

        Returns
        - True if the server should generate Vanilla noise

        See
        - .shouldGenerateNoise()
        """
        ...


    def shouldGenerateSurface(self) -> bool:
        """
        Gets if the server should generate Vanilla surface.
        
        The Vanilla surface is generated **before**
        .generateSurface(WorldInfo, Random, int, int, ChunkData) is
        called.
        
        This is method is not called (and has therefore no effect), if
        .shouldGenerateSurface(WorldInfo, Random, int, int) is overridden.

        Returns
        - True if the server should generate Vanilla surface

        See
        - .shouldGenerateSurface(WorldInfo, Random, int, int)
        """
        ...


    def shouldGenerateSurface(self, worldInfo: "WorldInfo", random: "Random", chunkX: int, chunkZ: int) -> bool:
        """
        Gets if the server should generate Vanilla surface.
        
        The Vanilla surface is generated **before**
        .generateSurface(WorldInfo, Random, int, int, ChunkData) is
        called.
        
        Only this method is called if both this and
        .shouldGenerateSurface() are overridden.

        Arguments
        - worldInfo: The world info of the world this chunk will be used for
        - random: The random generator to use
        - chunkX: The X-coordinate of the chunk
        - chunkZ: The Z-coordinate of the chunk

        Returns
        - True if the server should generate Vanilla surface

        See
        - .shouldGenerateSurface()
        """
        ...


    def shouldGenerateBedrock(self) -> bool:
        """
        Gets if the server should generate Vanilla bedrock.
        
        The Vanilla bedrock is generated **before**
        .generateBedrock(WorldInfo, Random, int, int, ChunkData) is
        called.

        Returns
        - True if the server should generate Vanilla bedrock

        Deprecated
        - has no effect, bedrock generation is part of the surface step, see .shouldGenerateSurface()
        """
        ...


    def shouldGenerateCaves(self) -> bool:
        """
        Gets if the server should generate Vanilla caves.
        
        The Vanilla caves are generated **before**
        .generateCaves(WorldInfo, Random, int, int, ChunkData) is called.
        
        This is method is not called (and has therefore no effect), if
        .shouldGenerateCaves(WorldInfo, Random, int, int) is overridden.

        Returns
        - True if the server should generate Vanilla caves

        See
        - .shouldGenerateCaves(WorldInfo, Random, int, int)
        """
        ...


    def shouldGenerateCaves(self, worldInfo: "WorldInfo", random: "Random", chunkX: int, chunkZ: int) -> bool:
        """
        Gets if the server should generate Vanilla caves.
        
        The Vanilla caves are generated **before**
        .generateCaves(WorldInfo, Random, int, int, ChunkData) is called.
        
        Only this method is called if both this and
        .shouldGenerateCaves() are overridden.

        Arguments
        - worldInfo: The world info of the world this chunk will be used for
        - random: The random generator to use
        - chunkX: The X-coordinate of the chunk
        - chunkZ: The Z-coordinate of the chunk

        Returns
        - True if the server should generate Vanilla caves

        See
        - .shouldGenerateCaves()
        """
        ...


    def shouldGenerateDecorations(self) -> bool:
        """
        Gets if the server should generate Vanilla decorations after this
        ChunkGenerator.
        
        The Vanilla decoration are generated **before** any
        BlockPopulator are called.
        
        This is method is not called (and has therefore no effect), if
        .shouldGenerateDecorations(WorldInfo, Random, int, int) is overridden.

        Returns
        - True if the server should generate Vanilla decorations

        See
        - .shouldGenerateDecorations(WorldInfo, Random, int, int)
        """
        ...


    def shouldGenerateDecorations(self, worldInfo: "WorldInfo", random: "Random", chunkX: int, chunkZ: int) -> bool:
        """
        Gets if the server should generate Vanilla decorations after this
        ChunkGenerator.
        
        The Vanilla decoration are generated **before** any
        BlockPopulator are called.
        
        Only this method is called if both this and
        .shouldGenerateDecorations() are overridden.

        Arguments
        - worldInfo: The world info of the world this chunk will be used for
        - random: The random generator to use
        - chunkX: The X-coordinate of the chunk
        - chunkZ: The Z-coordinate of the chunk

        Returns
        - True if the server should generate Vanilla decorations

        See
        - .shouldGenerateDecorations()
        """
        ...


    def shouldGenerateMobs(self) -> bool:
        """
        Gets if the server should generate Vanilla mobs after this
        ChunkGenerator.
        
        This is method is not called (and has therefore no effect), if
        .shouldGenerateMobs(WorldInfo, Random, int, int) is overridden.

        Returns
        - True if the server should generate Vanilla mobs

        See
        - .shouldGenerateMobs(WorldInfo, Random, int, int)
        """
        ...


    def shouldGenerateMobs(self, worldInfo: "WorldInfo", random: "Random", chunkX: int, chunkZ: int) -> bool:
        """
        Gets if the server should generate Vanilla mobs after this
        ChunkGenerator.
        
        Only this method is called if both this and
        .shouldGenerateMobs() are overridden.

        Arguments
        - worldInfo: The world info of the world this chunk will be used for
        - random: The random generator to use
        - chunkX: The X-coordinate of the chunk
        - chunkZ: The Z-coordinate of the chunk

        Returns
        - True if the server should generate Vanilla mobs

        See
        - .shouldGenerateMobs()
        """
        ...


    def shouldGenerateStructures(self) -> bool:
        """
        Gets if the server should generate Vanilla structures after this
        ChunkGenerator.
        
        This is method is not called (and has therefore no effect), if
        .shouldGenerateStructures(WorldInfo, Random, int, int) is overridden.

        Returns
        - True if the server should generate Vanilla structures

        See
        - .shouldGenerateStructures(WorldInfo, Random, int, int)
        """
        ...


    def shouldGenerateStructures(self, worldInfo: "WorldInfo", random: "Random", chunkX: int, chunkZ: int) -> bool:
        """
        Gets if the server should generate Vanilla structures after this
        ChunkGenerator.
        
        Only this method is called if both this and
        .shouldGenerateStructures() are overridden.

        Arguments
        - worldInfo: The world info of the world this chunk will be used for
        - random: The random generator to use
        - chunkX: The X-coordinate of the chunk
        - chunkZ: The Z-coordinate of the chunk

        Returns
        - True if the server should generate Vanilla structures

        See
        - .shouldGenerateStructures()
        """
        ...


    class BiomeGrid:
        """
        Interface to biome section for chunk to be generated: initialized with
        default values for world type and seed.
        
        Custom generator is free to access and tailor values during
        generateBlockSections() or generateExtBlockSections().

        Deprecated
        - Biomes are now set with BiomeProvider
        """

        def getBiome(self, x: int, z: int) -> "Biome":
            """
            Get biome at x, z within chunk being generated

            Arguments
            - x: - 0-15
            - z: - 0-15

            Returns
            - Biome value

            Deprecated
            - biomes are now 3-dimensional
            """
            ...


        def getBiome(self, x: int, y: int, z: int) -> "Biome":
            """
            Get biome at x, z within chunk being generated

            Arguments
            - x: - 0-15
            - y: - world minHeight (inclusive) - world maxHeight (exclusive)
            - z: - 0-15

            Returns
            - Biome value
            """
            ...


        def setBiome(self, x: int, z: int, bio: "Biome") -> None:
            """
            Set biome at x, z within chunk being generated

            Arguments
            - x: - 0-15
            - z: - 0-15
            - bio: - Biome value

            Deprecated
            - biomes are now 3-dimensional
            """
            ...


        def setBiome(self, x: int, y: int, z: int, bio: "Biome") -> None:
            """
            Set biome at x, z within chunk being generated

            Arguments
            - x: - 0-15
            - y: - world minHeight (inclusive) - world maxHeight (exclusive)
            - z: - 0-15
            - bio: - Biome value
            """
            ...


    class ChunkData:
        """
        Data for a Chunk.
        """

        def getMinHeight(self) -> int:
            """
            Get the minimum height for this ChunkData.
            
            It is not guaranteed that this method will return the same value as
            World.getMinHeight().
            
            Setting blocks below this height will do nothing.

            Returns
            - the minimum height
            """
            ...


        def getMaxHeight(self) -> int:
            """
            Get the maximum height for this ChunkData.
            
            It is not guaranteed that this method will return the same value as
            World.getMaxHeight().
            
            Setting blocks at or above this height will do nothing.

            Returns
            - the maximum height
            """
            ...


        def getBiome(self, x: int, y: int, z: int) -> "Biome":
            """
            Get the biome at x, y, z within chunk being generated

            Arguments
            - x: the x location in the chunk from 0-15 inclusive
            - y: the y location in the chunk from minimum (inclusive) -
            maxHeight (exclusive)
            - z: the z location in the chunk from 0-15 inclusive

            Returns
            - Biome value
            """
            ...


        def setBlock(self, x: int, y: int, z: int, material: "Material") -> None:
            """
            Set the block at x,y,z in the chunk data to material.
            
            Note: setting blocks outside the chunk's bounds does nothing.

            Arguments
            - x: the x location in the chunk from 0-15 inclusive
            - y: the y location in the chunk from minHeight (inclusive) - maxHeight (exclusive)
            - z: the z location in the chunk from 0-15 inclusive
            - material: the type to set the block to
            """
            ...


        def setBlock(self, x: int, y: int, z: int, material: "MaterialData") -> None:
            """
            Set the block at x,y,z in the chunk data to material.
            
            Setting blocks outside the chunk's bounds does nothing.

            Arguments
            - x: the x location in the chunk from 0-15 inclusive
            - y: the y location in the chunk from minHeight (inclusive) - maxHeight (exclusive)
            - z: the z location in the chunk from 0-15 inclusive
            - material: the type to set the block to
            """
            ...


        def setBlock(self, x: int, y: int, z: int, blockData: "BlockData") -> None:
            """
            Set the block at x,y,z in the chunk data to material.
            
            Setting blocks outside the chunk's bounds does nothing.

            Arguments
            - x: the x location in the chunk from 0-15 inclusive
            - y: the y location in the chunk from minHeight (inclusive) - maxHeight (exclusive)
            - z: the z location in the chunk from 0-15 inclusive
            - blockData: the type to set the block to
            """
            ...


        def setRegion(self, xMin: int, yMin: int, zMin: int, xMax: int, yMax: int, zMax: int, material: "Material") -> None:
            """
            Set a region of this chunk from xMin, yMin, zMin (inclusive)
            to xMax, yMax, zMax (exclusive) to material.
            
            Setting blocks outside the chunk's bounds does nothing.

            Arguments
            - xMin: minimum x location (inclusive) in the chunk to set
            - yMin: minimum y location (inclusive) in the chunk to set
            - zMin: minimum z location (inclusive) in the chunk to set
            - xMax: maximum x location (exclusive) in the chunk to set
            - yMax: maximum y location (exclusive) in the chunk to set
            - zMax: maximum z location (exclusive) in the chunk to set
            - material: the type to set the blocks to
            """
            ...


        def setRegion(self, xMin: int, yMin: int, zMin: int, xMax: int, yMax: int, zMax: int, material: "MaterialData") -> None:
            """
            Set a region of this chunk from xMin, yMin, zMin (inclusive)
            to xMax, yMax, zMax (exclusive) to material.
            
            Setting blocks outside the chunk's bounds does nothing.

            Arguments
            - xMin: minimum x location (inclusive) in the chunk to set
            - yMin: minimum y location (inclusive) in the chunk to set
            - zMin: minimum z location (inclusive) in the chunk to set
            - xMax: maximum x location (exclusive) in the chunk to set
            - yMax: maximum y location (exclusive) in the chunk to set
            - zMax: maximum z location (exclusive) in the chunk to set
            - material: the type to set the blocks to
            """
            ...


        def setRegion(self, xMin: int, yMin: int, zMin: int, xMax: int, yMax: int, zMax: int, blockData: "BlockData") -> None:
            """
            Set a region of this chunk from xMin, yMin, zMin (inclusive) to xMax,
            yMax, zMax (exclusive) to material.
            
            Setting blocks outside the chunk's bounds does nothing.

            Arguments
            - xMin: minimum x location (inclusive) in the chunk to set
            - yMin: minimum y location (inclusive) in the chunk to set
            - zMin: minimum z location (inclusive) in the chunk to set
            - xMax: maximum x location (exclusive) in the chunk to set
            - yMax: maximum y location (exclusive) in the chunk to set
            - zMax: maximum z location (exclusive) in the chunk to set
            - blockData: the type to set the blocks to
            """
            ...


        def getType(self, x: int, y: int, z: int) -> "Material":
            """
            Get the type of the block at x, y, z.
            
            Getting blocks outside the chunk's bounds returns air.

            Arguments
            - x: the x location in the chunk from 0-15 inclusive
            - y: the y location in the chunk from minHeight (inclusive) - maxHeight (exclusive)
            - z: the z location in the chunk from 0-15 inclusive

            Returns
            - the type of the block or Material.AIR if x, y or z are outside the chunk's bounds
            """
            ...


        def getTypeAndData(self, x: int, y: int, z: int) -> "MaterialData":
            """
            Get the type and data of the block at x, y, z.
            
            Getting blocks outside the chunk's bounds returns air.

            Arguments
            - x: the x location in the chunk from 0-15 inclusive
            - y: the y location in the chunk from minHeight (inclusive) - maxHeight (exclusive)
            - z: the z location in the chunk from 0-15 inclusive

            Returns
            - the type and data of the block or the MaterialData for air if x, y or z are outside the chunk's bounds
            """
            ...


        def getBlockData(self, x: int, y: int, z: int) -> "BlockData":
            """
            Get the type and data of the block at x, y, z.
            
            Getting blocks outside the chunk's bounds returns air.

            Arguments
            - x: the x location in the chunk from 0-15 inclusive
            - y: the y location in the chunk from minHeight (inclusive) - maxHeight (exclusive)
            - z: the z location in the chunk from 0-15 inclusive

            Returns
            - the data of the block or the BlockData for air if x, y or z are outside the chunk's bounds
            """
            ...


        def getData(self, x: int, y: int, z: int) -> int:
            """
            Get the block data at x,y,z in the chunk data.
            
            Getting blocks outside the chunk's bounds returns 0.

            Arguments
            - x: the x location in the chunk from 0-15 inclusive
            - y: the y location in the chunk from minHeight (inclusive) - maxHeight (exclusive)
            - z: the z location in the chunk from 0-15 inclusive

            Returns
            - the block data value or air if x, y or z are outside the chunk's bounds

            Deprecated
            - Uses magic values
            """
            ...



"""
Python module generated from Java source file org.bukkit.inventory.view.builder.LocationInventoryViewBuilder

Java source file obtained from artifact spigot-api version 1.21.4-R0.1-20250303.102353-42

Because this Python module is automatically generated, it may contain errors
and/or code that cannot be parsed. Please report these issues at
https://github.com/magicmq/docs-translator/issues
"""
from org.bukkit import Location
from org.bukkit.inventory import InventoryView
from org.bukkit.inventory.view.builder import *
from typing import Any, Callable, Iterable, Tuple


class LocationInventoryViewBuilder(InventoryViewBuilder):
    """
    An InventoryViewBuilder that can be bound by location within the world
    
    Type `<V>`: the type of InventoryView created from this builder
    """

    def copy(self) -> "LocationInventoryViewBuilder"["V"]:
        ...


    def title(self, title: str) -> "LocationInventoryViewBuilder"["V"]:
        ...


    def checkReachable(self, checkReachable: bool) -> "LocationInventoryViewBuilder"["V"]:
        """
        Determines whether or not the server should check if the player can reach
        the location.
        
        Not providing a location but setting checkReachable to True will
        automatically close the view when opened.
        
        If checkReachable is set to False and a location is set on the builder if
        the target block exists and this builder is the correct menu for that
        block, e.g. MenuType.GENERIC_9X3 builder and target block set to chest,
        if that block is destroyed the view would persist.

        Arguments
        - checkReachable: whether or not to check if the view is "reachable"

        Returns
        - this builder
        """
        ...


    def location(self, location: "Location") -> "LocationInventoryViewBuilder"["V"]:
        """
        Binds a location to this builder.
        
        By binding a location in an unloaded chunk to this builder it is likely
        that the given chunk the location is will load. That means that when,
        building this view it may come with the costs associated with chunk
        loading.
        
        Providing a location of a tile entity with a non matching menu comes with
        extra costs associated with ensuring that the correct view is created.

        Arguments
        - location: the location to bind to this view

        Returns
        - this builder
        """
        ...



"""
Python module generated from Java source file org.bukkit.entity.Display

Java source file obtained from artifact spigot-api version 1.21.4-R0.1-20250303.102353-42

Because this Python module is automatically generated, it may contain errors
and/or code that cannot be parsed. Please report these issues at
https://github.com/magicmq/docs-translator/issues
"""
from com.google.common.base import Preconditions
from enum import Enum
from org.bukkit import Color
from org.bukkit.entity import *
from org.bukkit.util import Transformation
from org.joml import Matrix4f
from typing import Any, Callable, Iterable, Tuple


class Display(Entity):
    """
    Represents a display entity which is designed to only have a visual function.
    """

    def getTransformation(self) -> "Transformation":
        """
        Gets the transformation applied to this display.

        Returns
        - the transformation
        """
        ...


    def setTransformation(self, transformation: "Transformation") -> None:
        """
        Sets the transformation applied to this display

        Arguments
        - transformation: the new transformation
        """
        ...


    def setTransformationMatrix(self, transformationMatrix: "Matrix4f") -> None:
        """
        Sets the raw transformation matrix applied to this display

        Arguments
        - transformationMatrix: the transformation matrix
        """
        ...


    def getInterpolationDuration(self) -> int:
        """
        Gets the interpolation duration of this display.

        Returns
        - interpolation duration
        """
        ...


    def setInterpolationDuration(self, duration: int) -> None:
        """
        Sets the interpolation duration of this display.

        Arguments
        - duration: new duration
        """
        ...


    def getTeleportDuration(self) -> int:
        """
        Gets the teleport duration of this display.
        
            - 0 means that updates are applied immediately.
            - 1 means that the display entity will move from current position to the updated one over one tick.
            - Higher values spread the movement over multiple ticks.

        Returns
        - teleport duration
        """
        ...


    def setTeleportDuration(self, duration: int) -> None:
        """
        Sets the teleport duration of this display.

        Arguments
        - duration: new duration

        Raises
        - IllegalArgumentException: if duration is not between 0 and 59

        See
        - .getTeleportDuration()
        """
        ...


    def getViewRange(self) -> float:
        """
        Gets the view distance/range of this display.

        Returns
        - view range
        """
        ...


    def setViewRange(self, range: float) -> None:
        """
        Sets the view distance/range of this display.

        Arguments
        - range: new range
        """
        ...


    def getShadowRadius(self) -> float:
        """
        Gets the shadow radius of this display.

        Returns
        - radius
        """
        ...


    def setShadowRadius(self, radius: float) -> None:
        """
        Sets the shadow radius of this display.

        Arguments
        - radius: new radius
        """
        ...


    def getShadowStrength(self) -> float:
        """
        Gets the shadow strength of this display.

        Returns
        - shadow strength
        """
        ...


    def setShadowStrength(self, strength: float) -> None:
        """
        Sets the shadow strength of this display.

        Arguments
        - strength: new strength
        """
        ...


    def getDisplayWidth(self) -> float:
        """
        Gets the width of this display.

        Returns
        - width
        """
        ...


    def setDisplayWidth(self, width: float) -> None:
        """
        Sets the width of this display.

        Arguments
        - width: new width
        """
        ...


    def getDisplayHeight(self) -> float:
        """
        Gets the height of this display.

        Returns
        - height
        """
        ...


    def setDisplayHeight(self, height: float) -> None:
        """
        Sets the height if this display.

        Arguments
        - height: new height
        """
        ...


    def getInterpolationDelay(self) -> int:
        """
        Gets the amount of ticks before client-side interpolation will commence.

        Returns
        - interpolation delay ticks
        """
        ...


    def setInterpolationDelay(self, ticks: int) -> None:
        """
        Sets the amount of ticks before client-side interpolation will commence.

        Arguments
        - ticks: interpolation delay ticks
        """
        ...


    def getBillboard(self) -> "Billboard":
        """
        Gets the billboard setting of this entity.
        
        The billboard setting controls the automatic rotation of the entity to
        face the player.

        Returns
        - billboard setting
        """
        ...


    def setBillboard(self, billboard: "Billboard") -> None:
        """
        Sets the billboard setting of this entity.
        
        The billboard setting controls the automatic rotation of the entity to
        face the player.

        Arguments
        - billboard: new setting
        """
        ...


    def getGlowColorOverride(self) -> "Color":
        """
        Gets the scoreboard team overridden glow color of this display.

        Returns
        - glow color
        """
        ...


    def setGlowColorOverride(self, color: "Color") -> None:
        """
        Sets the scoreboard team overridden glow color of this display.

        Arguments
        - color: new color
        """
        ...


    def getBrightness(self) -> "Brightness":
        """
        Gets the brightness override of the entity.

        Returns
        - brightness override, if set
        """
        ...


    def setBrightness(self, brightness: "Brightness") -> None:
        """
        Sets the brightness override of the entity.

        Arguments
        - brightness: new brightness override
        """
        ...


    class Brightness:
        """
        Represents the brightness rendering parameters of the entity.
        """

        def __init__(self, blockLight: int, skyLight: int):
            ...


        def getBlockLight(self) -> int:
            """
            Gets the block lighting component of this brightness.

            Returns
            - block light, between 0-15
            """
            ...


        def getSkyLight(self) -> int:
            """
            Gets the sky lighting component of this brightness.

            Returns
            - sky light, between 0-15
            """
            ...


        def hashCode(self) -> int:
            ...


        def equals(self, obj: "Object") -> bool:
            ...


        def toString(self) -> str:
            ...


    class Billboard(Enum):
        """
        Describes the axes/points around which the entity can pivot.
        """

        FIXED = 0
        """
        No rotation (default).
        """
        VERTICAL = 1
        """
        Can pivot around vertical axis.
        """
        HORIZONTAL = 2
        """
        Can pivot around horizontal axis.
        """
        CENTER = 3
        """
        Can pivot around center point.
        """



"""
Python module generated from Java source file org.bukkit.event.entity.EntityPortalExitEvent

Java source file obtained from artifact spigot-api version 1.21.4-R0.1-20250303.102353-42

Because this Python module is automatically generated, it may contain errors
and/or code that cannot be parsed. Please report these issues at
https://github.com/magicmq/docs-translator/issues
"""
from org.bukkit import Location
from org.bukkit.entity import Entity
from org.bukkit.event import HandlerList
from org.bukkit.event.entity import *
from org.bukkit.util import Vector
from typing import Any, Callable, Iterable, Tuple


class EntityPortalExitEvent(EntityTeleportEvent):
    """
    Called before an entity exits a portal.
    
    This event allows you to modify the velocity of the entity after they have
    successfully exited the portal.
    """

    def __init__(self, entity: "Entity", from: "Location", to: "Location", before: "Vector", after: "Vector"):
        ...


    def getBefore(self) -> "Vector":
        """
        Gets a copy of the velocity that the entity has before entering the
        portal.

        Returns
        - velocity of entity before entering the portal
        """
        ...


    def getAfter(self) -> "Vector":
        """
        Gets a copy of the velocity that the entity will have after exiting the
        portal.

        Returns
        - velocity of entity after exiting the portal
        """
        ...


    def setAfter(self, after: "Vector") -> None:
        """
        Sets the velocity that the entity will have after exiting the portal.

        Arguments
        - after: the velocity after exiting the portal
        """
        ...


    def getHandlers(self) -> "HandlerList":
        ...


    @staticmethod
    def getHandlerList() -> "HandlerList":
        ...



"""
Python module generated from Java source file com.google.common.collect.EnumMultiset

Java source file obtained from artifact guava version 33.3.1-jre

Because this Python module is automatically generated, it may contain errors
and/or code that cannot be parsed. Please report these issues at
https://github.com/magicmq/docs-translator/issues
"""
from com.google.common.annotations import GwtCompatible
from com.google.common.annotations import GwtIncompatible
from com.google.common.annotations import J2ktIncompatible
from com.google.common.collect import *
from com.google.common.primitives import Ints
from com.google.errorprone.annotations import CanIgnoreReturnValue
from java.io import IOException
from java.io import ObjectInputStream
from java.io import ObjectOutputStream
from java.io import Serializable
from java.util import Arrays
from java.util import Iterator
from java.util import NoSuchElementException
from java.util.function import ObjIntConsumer
from javax.annotation import CheckForNull
from typing import Any, Callable, Iterable, Tuple


class EnumMultiset(AbstractMultiset, Serializable):
    """
    Multiset implementation specialized for enum elements, supporting all single-element operations
    in O(1).
    
    See the Guava User Guide article on <a href=
    "https://github.com/google/guava/wiki/NewCollectionTypesExplained#multiset">`Multiset`</a>.

    Author(s)
    - Jared Levy

    Since
    - 2.0
    """

    @staticmethod
    def create(type: type["E"]) -> "EnumMultiset"["E"]:
        """
        Creates an empty `EnumMultiset`.
        """
        ...


    @staticmethod
    def create(elements: Iterable["E"]) -> "EnumMultiset"["E"]:
        """
        Creates a new `EnumMultiset` containing the specified elements.
        
        This implementation is highly efficient when `elements` is itself a Multiset.

        Arguments
        - elements: the elements that the multiset should contain

        Raises
        - IllegalArgumentException: if `elements` is empty
        """
        ...


    @staticmethod
    def create(elements: Iterable["E"], type: type["E"]) -> "EnumMultiset"["E"]:
        """
        Returns a new `EnumMultiset` instance containing the given elements. Unlike EnumMultiset.create(Iterable), this method does not produce an exception on an empty iterable.

        Since
        - 14.0
        """
        ...


    def size(self) -> int:
        ...


    def count(self, element: "Object") -> int:
        ...


    def add(self, element: "E", occurrences: int) -> int:
        ...


    def remove(self, element: "Object", occurrences: int) -> int:
        ...


    def setCount(self, element: "E", count: int) -> int:
        ...


    def clear(self) -> None:
        ...


    def forEachEntry(self, action: "ObjIntConsumer"["E"]) -> None:
        ...


    def iterator(self) -> Iterator["E"]:
        ...



"""
Python module generated from Java source file com.google.common.collect.SortedIterable

Java source file obtained from artifact guava version 33.3.1-jre

Because this Python module is automatically generated, it may contain errors
and/or code that cannot be parsed. Please report these issues at
https://github.com/magicmq/docs-translator/issues
"""
from com.google.common.annotations import GwtCompatible
from com.google.common.collect import *
from java.util import Comparator
from java.util import Iterator
from org.checkerframework.checker.nullness.qual import Nullable
from typing import Any, Callable, Iterable, Tuple


class SortedIterable(Iterable):
    """
    An `Iterable` whose elements are sorted relative to a `Comparator`, typically
    provided at creation time.

    Author(s)
    - Louis Wasserman
    """

    def comparator(self) -> "Comparator"["T"]:
        """
        Returns the `Comparator` by which the elements of this iterable are ordered, or `Ordering.natural()` if the elements are ordered by their natural ordering.
        """
        ...


    def iterator(self) -> Iterator["T"]:
        """
        Returns an iterator over elements of type `T`. The elements are returned in nondecreasing
        order according to the associated .comparator.
        """
        ...
